"""Views to handle asynchronous tasks."""
import logging
from datetime import (
    timedelta,
)

from django.dispatch import receiver
from django.utils import timezone
from django.shortcuts import render
from django.conf import settings
from django.http import (
    HttpResponseRedirect,
    JsonResponse,
)
from django.views.decorators.http import require_http_methods

from django_q.tasks import async_task
from django_q.signals import post_execute

from mobsf.StaticAnalyzer.models import (
    EnqueuedTask,
    RecentScansDB,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.utils import (
    append_scan_status,
    get_scan_logs,
)

logger = logging.getLogger(__name__)


@receiver(post_execute)
def detect_timeout(sender, task, **kwargs):
    """Detect scan task timeout."""
    result = task.get('result')
    if isinstance(result, str) and 'Task exceeded maximum timeout' in result:
        task_id = task['id']
        EnqueuedTask.objects.filter(task_id=task_id).update(
            app_name='Failed',
            completed_at=timezone.now(),
            status='Scan Timeout',
        )
        logger.error('Task %s exceeded maximum timeout', task_id)


def async_analysis(checksum, api, file_name, func, *args, **kwargs):
    """Async Analysis Task."""
    # Check if the scan is already completed and successful
    scan_completed = False
    recent = RecentScansDB.objects.filter(MD5=checksum)
    if recent.exists() and (recent[0].APP_NAME or recent[0].PACKAGE_NAME):
        # Successful scan will have APP_NAME or PACKAGE_NAME
        scan_completed = True
    # Check if task is already completed within the last 60 minutes
    # Can be success or failed
    completed_at_recently = EnqueuedTask.objects.filter(
        checksum=checksum,
        completed_at__gte=timezone.now() - timedelta(
            minutes=settings.ASYNC_ANALYSIS_TIMEOUT),
    ).exists()

    # Check if the task is already enqueued within the last 60 minutes
    queued_recently = EnqueuedTask.objects.filter(
        checksum=checksum,
        created_at__gte=timezone.now() - timedelta(
            minutes=settings.ASYNC_ANALYSIS_TIMEOUT),
    ).exists()
    # Check if the task is already started within the last 60 minutes
    started_at_recently = EnqueuedTask.objects.filter(
        checksum=checksum,
        started_at__gte=timezone.now() - timedelta(
            minutes=settings.ASYNC_ANALYSIS_TIMEOUT),
    ).exists()

    # Prevent duplicate scans in the last 60 minutes
    if scan_completed and completed_at_recently:
        # scan task already completed with success/failure recently
        msg = ('Analysis already completed/failed in the '
               f'last {settings.ASYNC_ANALYSIS_TIMEOUT} minutes. '
               'Please try again later.')
        logger.warning(msg)
        if api:
            return {'task_id': None, 'message': msg}
        return HttpResponseRedirect('/tasks?q=completed')
    elif queued_recently or started_at_recently:
        # scan not completed but queued or started recently
        msg = ('Analysis already enqueued in the '
               f'last {settings.ASYNC_ANALYSIS_TIMEOUT} minutes. '
               'Please wait for the current scan to complete.')
        logger.warning(msg)
        if api:
            return {'task_id': None, 'message': msg}
        return HttpResponseRedirect('/tasks?q=queued')

    # Clear old tasks
    queue_size = settings.QUEUE_MAX_SIZE
    task_count = EnqueuedTask.objects.count()
    if task_count > queue_size:
        logger.info('Deleting oldest enqueued tasks')
        # Get IDs of tasks to delete (keep the latest queue_size)
        oldest_task_ids = list(
            EnqueuedTask.objects.order_by('created_at')
            .values_list('id', flat=True)[:task_count - queue_size])
        # Delete tasks by IDs
        EnqueuedTask.objects.filter(id__in=oldest_task_ids).delete()

    # Enqueue the task
    task_id = async_task(
        func,
        *args,
        queue=True,
        save=False)
    EnqueuedTask.objects.create(
        task_id=task_id,
        checksum=checksum,
        file_name=file_name[:254])
    msg = f'Scan Queued with ID: {task_id}'
    logger.info(msg)
    append_scan_status(checksum, msg)
    if api:
        return {'task_id': task_id, 'message': msg}
    return HttpResponseRedirect('/tasks')


def mark_task_started(checksum):
    """Register the enqued task and others that matches the checksum as started."""
    EnqueuedTask.objects.filter(checksum=checksum).update(
        started_at=timezone.now(),
    )


def mark_task_completed(checksum, app_name, status):
    """Update the enqueued task and others that matches the checksum as completed."""
    EnqueuedTask.objects.filter(checksum=checksum).update(
        app_name=app_name[:254],
        completed_at=timezone.now(),
        status=status[:254],
    )
    return True


def get_live_status(enq):
    """Get Live Status of the Task."""
    if enq.status == 'Success' or enq.app_name == 'Failed':
        return enq.status
    logs = get_scan_logs(enq.checksum)
    if logs:
        return logs[-1].get('status', '')
    return enq.status


@login_required
@require_http_methods(['POST', 'GET'])
def list_tasks(request, api=False):
    if request.method == 'POST':
        enqueued = EnqueuedTask.objects.all().order_by('-created_at')
        task_data = []
        for enq in enqueued:
            # Enqueued task is not in the completed tasks
            task_data.append({
                'task_id': enq.task_id,
                'file_name': enq.file_name,
                'app_name': enq.app_name,
                'checksum': enq.checksum,
                'created_at': enq.created_at,
                'started_at': enq.started_at,
                'completed_at': enq.completed_at,
                'status': get_live_status(enq),
            })
        if api:
            return task_data
        return JsonResponse(task_data, safe=False)
    context = {
        'title': 'Scan Tasks',
        'version': settings.MOBSF_VER,
    }
    return render(request, 'general/tasks.html', context)




# -*- coding: utf_8 -*-
"""Entropy Scanner."""
import re
import math

MAX_LENGTH = 21
ENTROPY_PATTERNS = [
    {
        # Base64
        'pattern': re.compile(r'(?m)[a-zA-Z\d+/=]{20,}'),
        'charset': ('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdef'
                    'ghijklmnopqrstuvwxyz0123456789+/='),
        'score': 4.5,
    },
    {
        # Hex
        'pattern': re.compile(r'(?m)[a-fA-F\d-]{20,}'),
        'charset': '1234567890abcdefABCDEF',
        'score': 3.0,
    },
]


def entropy(data, charset):
    """Shannon Entropy score calculations."""
    # Based on
    # http://blog.dkbza.org/2007/05/
    # scanning-data-for-entropy-anomalies.html
    entropy = 0
    for char in charset:
        p_x = float(data.count(char)) / len(data)
        if p_x > 0:
            entropy += -p_x * math.log2(p_x)
    return entropy


def exclude(secret):
    """Exclude entropies."""
    excludes = ('abcdefghi', 'kotlin/')
    if secret.startswith('L') and '/' in secret:
        return True
    if any(i in secret.lower() for i in excludes):
        return True
    if secret.count('/') > 1:
        # URLs/paths, ignore them
        return True
    if secret.isalpha():
        return True
    return False


def get_entropies(data):
    patterns = set()
    for ascii_string in data:
        if len(ascii_string) < MAX_LENGTH:
            continue
        for p in ENTROPY_PATTERNS:
            for i in re.findall(p['pattern'], ascii_string):
                score = p['score']
                if entropy(i, p['charset']) > score and not exclude(i):
                    patterns.add(i)
    return patterns



#!/usr/bin/env python
import json
import logging
import os
import platform

from mobsf.MobSF.init import api_key

from django.conf import settings
from django.http import HttpResponse
from django.test import Client, TestCase

logger = logging.getLogger(__name__)

RESCAN = False
# Set RESCAN to True if Static Analyzer Code is modified
EXTS = settings.ANDROID_EXTS + settings.IOS_EXTS + settings.WINDOWS_EXTS


def static_analysis_test():
    """Test Static Analyzer."""
    logger.info('Running Static Analyzer Unit test')
    try:
        uploaded = []
        logger.info('Running Upload Test')
        http_client = Client()
        apk_dir = os.path.join(settings.BASE_DIR, 'StaticAnalyzer/test_files/')
        for filename in os.listdir(apk_dir):
            if not filename.endswith(EXTS):
                continue
            if platform.system() == 'Windows' and filename.endswith('.ipa'):
                continue
            fpath = os.path.join(apk_dir, filename)
            with open(fpath, 'rb') as file_pointer:
                response = http_client.post(
                    '/upload/',
                    {'file': file_pointer})
                obj = json.loads(response.content.decode('utf-8'))
                if response.status_code == 200 and obj['status'] == 'success':
                    logger.info('[OK] Upload OK: %s', filename)
                    uploaded.append(obj)
                else:
                    logger.error('Performing Upload: %s', filename)
                    return True
        logger.info('[OK] Completed Upload test')
        logger.info('Running Static Analysis Test')
        for upl in uploaded:
            scan_url = '/{}/{}/'.format(
                upl['analyzer'],
                upl['hash'])
            if RESCAN:
                scan_url = scan_url + '?rescan=1'
            resp = http_client.get(scan_url, follow=True)
            if resp.status_code == 200:
                logger.info('[OK] Static Analysis Complete: %s', scan_url)
            else:
                logger.error('Performing Static Analysis: %s', scan_url)
                return True
        logger.info('[OK] Static Analysis test completed')
        logger.info('Running PDF Generation Test')
        if platform.system() in ['Darwin', 'Linux']:
            pdfs = [
                '/pdf/02e7989c457ab67eb514a8328779f256/',
                '/pdf/82ab8b2193b3cfb1c737e3a786be363a/',
                '/pdf/6c23c2970551be15f32bbab0b5db0c71/',
                '/pdf/52c50ae824e329ba8b5b7a0f523efffe/',
                '/pdf/57bb5be0ea44a755ada4a93885c3825e/',
                '/pdf/8179b557433835827a70510584f3143e/',
                '/pdf/7b0a23bffc80bac05739ea1af898daad/',
            ]
        else:
            pdfs = [
                '/pdf/02e7989c457ab67eb514a8328779f256/',
                '/pdf/82ab8b2193b3cfb1c737e3a786be363a/',
                '/pdf/52c50ae824e329ba8b5b7a0f523efffe/',
                '/pdf/57bb5be0ea44a755ada4a93885c3825e/',
                '/pdf/8179b557433835827a70510584f3143e/',
                '/pdf/7b0a23bffc80bac05739ea1af898daad/',
            ]

        for pdf in pdfs:
            resp = http_client.get(pdf)
            if (resp.status_code == 200
                    and resp.headers['content-type'] == 'application/pdf'):
                logger.info('[OK] PDF Report Generated: %s', pdf)
            else:
                logger.error('Generating PDF: %s', pdf)
                logger.info(resp.content)
                return True
        logger.info('[OK] PDF Generation test completed')

        # Compare apps test
        logger.info('Running App Compare tests')
        first_app = '82ab8b2193b3cfb1c737e3a786be363a'
        second_app = '52c50ae824e329ba8b5b7a0f523efffe'
        url = '/compare/{}/{}/'.format(first_app, second_app)
        resp = http_client.get(url, follow=True)
        assert (resp.status_code == 200)
        if resp.status_code == 200:
            logger.info('[OK] App compare tests passed successfully')
        else:
            logger.error('App compare tests failed')
            logger.info(resp.content)
            return True

        # Scan shared object or dylib from binaries.
        logger.info('Running Library Analysis test')
        md5 = '82ab8b2193b3cfb1c737e3a786be363a'
        lib = 'apktool_out/lib/arm64-v8a/libdivajni.so'
        url = f'/scan_library/{md5}?library={lib}'
        resp = http_client.get(url, follow=True)
        assert (resp.status_code == 200)
        if resp.status_code == 200:
            logger.info('[OK] Library Analysis test passed successfully')
        else:
            logger.error('Library Analysis test failed')
            logger.info(resp.content)
            return True

        # Search by MD5 and text
        if platform.system() in ['Darwin', 'Linux']:
            scan_md5s = ['02e7989c457ab67eb514a8328779f256',
                         '82ab8b2193b3cfb1c737e3a786be363a',
                         '6c23c2970551be15f32bbab0b5db0c71',
                         '52c50ae824e329ba8b5b7a0f523efffe',
                         '57bb5be0ea44a755ada4a93885c3825e',
                         '8179b557433835827a70510584f3143e',
                         '7b0a23bffc80bac05739ea1af898daad']
        else:
            scan_md5s = ['02e7989c457ab67eb514a8328779f256',
                         '82ab8b2193b3cfb1c737e3a786be363a',
                         '52c50ae824e329ba8b5b7a0f523efffe',
                         '57bb5be0ea44a755ada4a93885c3825e',
                         '8179b557433835827a70510584f3143e',
                         '7b0a23bffc80bac05739ea1af898daad']
        # Search by text
        queries = [
            'diva',
            'webview',
        ]
        logger.info('Running Search test')
        for q in scan_md5s + queries:
            url = f'/search?query={q}'
            resp = http_client.get(url, follow=True)
            assert (resp.status_code == 200)
            if resp.status_code == 200:
                logger.info('[OK] Search by query test passed for %s', q)
            else:
                logger.error('Search by query test failed for %s', q)
                logger.info(resp.content)
                return True
        logger.info('[OK] Search by MD5 and text tests completed')

        # Deleting Scan Results
        logger.info('Running Delete Scan Results test')
        for md5 in scan_md5s:
            resp = http_client.post('/delete_scan/', {'md5': md5})
            if resp.status_code == 200:
                dat = json.loads(resp.content.decode('utf-8'))
                if dat['deleted'] == 'yes':
                    logger.info('[OK] Deleted Scan: %s', md5)
                else:
                    logger.error('Deleting Scan: %s', md5)
                    return True
            else:
                logger.error('Deleting Scan: %s', md5)
                return True
        logger.info('Delete Scan Results test completed')
    except Exception:
        logger.exception('Completing Static Analyzer Test')
        return True
    return False


def api_test():
    """View for Handling REST API Test."""
    logger.info('\nRunning REST API Unit test')
    auth = api_key(settings.MOBSF_HOME)
    try:
        uploaded = []
        logger.info('Running Test on Upload API')
        http_client = Client()
        apk_dir = os.path.join(settings.BASE_DIR, 'StaticAnalyzer/test_files/')
        for filename in os.listdir(apk_dir):
            if not filename.endswith(EXTS):
                continue
            if platform.system() == 'Windows' and filename.endswith('.ipa'):
                continue
            fpath = os.path.join(apk_dir, filename)
            if (platform.system() not in ['Darwin', 'Linux']
                    and fpath.endswith('.ipa')):
                continue
            with open(fpath, 'rb') as file_pointer:
                response = http_client.post(
                    '/api/v1/upload',
                    {'file': file_pointer},
                    HTTP_AUTHORIZATION=auth)
                obj = json.loads(response.content.decode('utf-8'))
                if response.status_code == 200 and 'hash' in obj:
                    logger.info('[OK] Upload OK: %s', filename)
                    uploaded.append(obj)
                else:
                    logger.error('Performing Upload %s', filename)
                    return True
        logger.info('[OK] Completed Upload API test')
        logger.info('Running Static Analysis API Test')
        for upl in uploaded:
            resp = http_client.post(
                '/api/v1/scan',
                {'hash': upl['hash']},
                HTTP_AUTHORIZATION=auth)
            if resp.status_code == 200:
                logger.info('[OK] Static Analysis Complete: %s',
                            upl['file_name'])
            else:
                logger.error('Performing Static Analysis: %s',
                             upl['file_name'])
                return True
        logger.info('[OK] Static Analysis API test completed')
        # Scan List API test
        logger.info('Running Scan List API tests')
        resp = http_client.get('/api/v1/scans', HTTP_AUTHORIZATION=auth)
        if resp.status_code == 200:
            logger.info('Scan List API Test 1 success')
        else:
            logger.error('Scan List API Test 1')
            return True
        resp = http_client.get(
            '/api/v1/scans?page=1&page_size=10', HTTP_AUTHORIZATION=auth)
        if resp.status_code == 200:
            logger.info('Scan List API Test 2 success')
        else:
            logger.error('Scan List API Test 2')
            return True
        resp = http_client.get('/api/v1/scans', HTTP_X_MOBSF_API_KEY=auth)
        if resp.status_code == 200:
            logger.info('Scan List API Test with custom http header 1 success')
        else:
            logger.error('Scan List API Test with custom http header 1')
            return True
        resp = http_client.get(
            '/api/v1/scans?page=1&page_size=10', HTTP_X_MOBSF_API_KEY=auth)
        if resp.status_code == 200:
            logger.info('Scan List API Test with custom http header 2 success')
        else:
            logger.error('Scan List API Test with custom http header 2')
            return True
        logger.info('[OK] Scan List API tests completed')
        # Scan logs tests
        logger.info('Running Scan Logs API tests')
        for upl in uploaded:
            resp = http_client.post(
                '/api/v1/scan_logs',
                {'hash': upl['hash']},
                HTTP_AUTHORIZATION=auth)
            if resp.status_code == 200:
                logs = json.loads(resp.content.decode('utf-8'))
                if 'logs' in logs and len(logs['logs']) > 0:
                    logger.info('[OK] Scan Logs API test: %s', upl['hash'])
            else:
                logger.error('Scan Logs API test: %s', upl['hash'])
                return True
        logger.info('[OK] Static Analysis API test completed')
        # Search API Tests
        logger.info('Running Search API tests')
        for term in ['diva', 'webview', '52c50ae824e329ba8b5b7a0f523efffe']:
            resp = http_client.post(
                '/api/v1/search',
                {'query': term},
                HTTP_AUTHORIZATION=auth)
            if resp.status_code == 200:
                logger.info('[OK] Search API test: %s', term)
            else:
                logger.error('Search API test: %s', term)
                return True
        # PDF Tests
        logger.info('Running PDF Generation API Test')
        if platform.system() in ['Darwin', 'Linux']:
            pdfs = [
                {'hash': '02e7989c457ab67eb514a8328779f256'},
                {'hash': '82ab8b2193b3cfb1c737e3a786be363a'},
                {'hash': '6c23c2970551be15f32bbab0b5db0c71'},
                {'hash': '52c50ae824e329ba8b5b7a0f523efffe'},
                {'hash': '57bb5be0ea44a755ada4a93885c3825e'},
                {'hash': '8179b557433835827a70510584f3143e'},
                {'hash': '7b0a23bffc80bac05739ea1af898daad'},
            ]
        else:
            pdfs = [
                {'hash': '02e7989c457ab67eb514a8328779f256'},
                {'hash': '82ab8b2193b3cfb1c737e3a786be363a'},
                {'hash': '52c50ae824e329ba8b5b7a0f523efffe'},
                {'hash': '57bb5be0ea44a755ada4a93885c3825e'},
                {'hash': '8179b557433835827a70510584f3143e'},
                {'hash': '7b0a23bffc80bac05739ea1af898daad'},
            ]
        for pdf in pdfs:
            resp = http_client.post(
                '/api/v1/download_pdf', pdf, HTTP_AUTHORIZATION=auth)
            resp_custom = http_client.post(
                '/api/v1/download_pdf', pdf, HTTP_X_MOBSF_API_KEY=auth)
            assert (resp.status_code == 200)
            assert (resp_custom.status_code == 200)
            if (resp.status_code == 200
                    and resp.headers['content-type'] == 'application/pdf'):
                logger.info('[OK] PDF Report Generated: %s', pdf['hash'])
            else:
                logger.error('Generating PDF: %s', pdf['hash'])
                logger.info(resp.content)
                return True
        logger.info('[OK] PDF Generation API test completed')
        logger.info('Running JSON Report API test')
        # JSON Report
        ctype = 'application/json; charset=utf-8'
        for jsn in pdfs:
            resp = http_client.post(
                '/api/v1/report_json', jsn, HTTP_AUTHORIZATION=auth)
            resp_custom = http_client.post(
                '/api/v1/report_json', jsn, HTTP_X_MOBSF_API_KEY=auth)
            assert (resp.status_code == 200)
            assert (resp_custom.status_code == 200)
            if (resp.status_code == 200
                    and resp.headers['content-type'] == ctype):
                logger.info('[OK] JSON Report Generated: %s', jsn['hash'])
            else:
                logger.error('Generating JSON Response: %s', jsn['hash'])
                return True
        logger.info('[OK] JSON Report API test completed')
        logger.info('Running Scorecard API test')
        # Scorecard Report
        for scr in pdfs:
            if scr['hash'] == '8179b557433835827a70510584f3143e':
                # Windows Scorecard not yet implemented
                continue
            resp = http_client.post(
                '/api/v1/scorecard', scr, HTTP_AUTHORIZATION=auth)
            resp_custom = http_client.post(
                '/api/v1/scorecard', scr, HTTP_X_MOBSF_API_KEY=auth)
            if resp.status_code == 200 and resp_custom.status_code == 200:
                rp = json.loads(resp.content.decode('utf-8'))
                if 'security_score' in rp:
                    logger.info(
                        '[OK] Security Score - %s', rp['security_score'])
                else:
                    logger.error('Security Score Failed - %s', str(rp))
                    return True
            else:
                logger.error('Scorecard API Failed for - %s', scr['hash'])
                return True
        logger.info('[OK] Scorecard API test completed')
        logger.info('Running View Source API test')
        # View Source tests
        files = [{'file': 'jakhar/aseem/diva/MainActivity.java',
                  'type': 'apk',
                  'hash': '82ab8b2193b3cfb1c737e3a786be363a'},
                 {'file': 'opensecurity/webviewignoressl/MainActivity.java',
                  'type': 'studio',
                  'hash': '52c50ae824e329ba8b5b7a0f523efffe'},
                 {'file': 'DamnVulnerableIOSApp/AppDelegate.m',
                  'type': 'ios',
                  'hash': '57bb5be0ea44a755ada4a93885c3825e'}]
        if platform.system() in ['Darwin', 'Linux']:
            files.append({
                'file': 'helloworld.app/Info.plist',
                'type': 'ipa',
                'hash': '6c23c2970551be15f32bbab0b5db0c71'})
        for sfile in files:
            resp = http_client.post(
                '/api/v1/view_source', sfile, HTTP_AUTHORIZATION=auth)
            resp_custom = http_client.post(
                '/api/v1/view_source', sfile, HTTP_X_MOBSF_API_KEY=auth)
            assert (resp.status_code == 200)
            assert (resp_custom.status_code == 200)
            if resp.status_code == 200:
                dat = json.loads(resp.content.decode('utf-8'))
                if dat['title']:
                    logger.info('[OK] Reading - %s', sfile['file'])
                else:
                    logger.error('Reading - %s', sfile['file'])
                    return True
            else:
                logger.error('Reading - %s', sfile['file'])
                return True
        logger.info('[OK] View Source API test completed')
        # Compare apps test
        logger.info('Running App Compare API tests')
        resp = http_client.post(
            '/api/v1/compare',
            {
                'hash1': '82ab8b2193b3cfb1c737e3a786be363a',
                'hash2': '52c50ae824e329ba8b5b7a0f523efffe',
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        resp_custom = http_client.post(
            '/api/v1/compare',
            {
                'hash1': '82ab8b2193b3cfb1c737e3a786be363a',
                'hash2': '52c50ae824e329ba8b5b7a0f523efffe',
            },
            HTTP_X_MOBSF_API_KEY=auth)
        assert (resp_custom.status_code == 200)
        if resp.status_code == 200:
            logger.info('[OK] App compare API tests completed')
        else:
            logger.error('App compare API tests failed')
            logger.info(resp.content)
            return True
        logger.info('Running Delete Scan Results test')
        # Suppression tests
        # Android Manifest by rule
        and_hash = '82ab8b2193b3cfb1c737e3a786be363a'
        rule = 'app_is_debuggable'
        typ = 'manifest'
        logger.info('Running Suppression disable by rule for APK manifest')
        resp = http_client.post(
            '/api/v1/suppress_by_rule',
            {
                'hash': and_hash,
                'type': typ,
                'rule': rule,
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        dat = json.loads(resp.content.decode('utf-8'))
        if dat['status'] == 'ok':
            logger.info('[OK] Suppression by rule - %s', rule)
        else:
            logger.error('[ERROR] Suppression by rule - %s', rule)
            return True
        resp = http_client.post(
            '/api/v1/list_suppressions',
            {
                'hash': and_hash,
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        dat = resp.content.decode('utf-8')
        if rule in dat:
            logger.info('[OK] Listing suppression for - %s', and_hash)
        else:
            logger.error('[ERROR] Listing suppression for  - %s', and_hash)
            return True
        resp = http_client.post(
            '/api/v1/delete_suppression',
            {
                'hash': and_hash,
                'type': typ,
                'rule': rule,
                'kind': 'rule',
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        resp = http_client.post(
            '/api/v1/list_suppressions',
            {
                'hash': and_hash,
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        dat = resp.content.decode('utf-8')
        if rule not in dat:
            logger.info('[OK] Suppression deleted - %s', and_hash)
        else:
            logger.error('[ERROR] Suppression deletion - %s', and_hash)
            return True
        # iOS Code by Files
        ios_hash = '57bb5be0ea44a755ada4a93885c3825e'
        rule = 'ios_app_logging'
        typ = 'code'
        sfile = ('DamnVulnerableIOSApp/Cocoa'
                 'Lumberjack/DDAbstractDatabaseLogger.m')
        logger.info('Running Suppression by files for iOS ObjC source')
        resp = http_client.post(
            '/api/v1/suppress_by_files',
            {
                'hash': ios_hash,
                'type': typ,
                'rule': rule,
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        dat = json.loads(resp.content.decode('utf-8'))
        if dat['status'] == 'ok':
            logger.info('[OK] Suppression by files for - %s', rule)
        else:
            logger.error('[ERROR] Suppression by files for - %s', rule)
            return True
        resp = http_client.post(
            '/api/v1/list_suppressions',
            {
                'hash': ios_hash,
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        dat = resp.content.decode('utf-8')
        if rule in dat and sfile in dat:
            logger.info('[OK] Listing suppression for - %s', ios_hash)
        else:
            logger.error('[ERROR] Listing suppression for  - %s', ios_hash)
            return True
        resp = http_client.post(
            '/api/v1/delete_suppression',
            {
                'hash': ios_hash,
                'type': typ,
                'rule': rule,
                'kind': 'file',
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        resp = http_client.post(
            '/api/v1/list_suppressions',
            {
                'hash': ios_hash,
            },
            HTTP_AUTHORIZATION=auth)
        assert (resp.status_code == 200)
        dat = resp.content.decode('utf-8')
        if rule not in dat:
            logger.info('[OK] Suppression deleted - %s', ios_hash)
        else:
            logger.error('[ERROR] Suppression deletion - %s', ios_hash)
            return True
        # Deleting Scan Results
        if platform.system() in ['Darwin', 'Linux']:
            scan_md5s = ['02e7989c457ab67eb514a8328779f256',
                         '82ab8b2193b3cfb1c737e3a786be363a',
                         '6c23c2970551be15f32bbab0b5db0c71',
                         '52c50ae824e329ba8b5b7a0f523efffe',
                         '57bb5be0ea44a755ada4a93885c3825e',
                         '8179b557433835827a70510584f3143e',
                         '7b0a23bffc80bac05739ea1af898daad',
                         ]
        else:
            scan_md5s = ['02e7989c457ab67eb514a8328779f256',
                         '82ab8b2193b3cfb1c737e3a786be363a',
                         '52c50ae824e329ba8b5b7a0f523efffe',
                         '57bb5be0ea44a755ada4a93885c3825e',
                         '8179b557433835827a70510584f3143e',
                         '7b0a23bffc80bac05739ea1af898daad',
                         ]
        for md5 in scan_md5s:
            resp = http_client.post(
                '/api/v1/delete_scan', {'hash': md5}, HTTP_AUTHORIZATION=auth)
            if resp.status_code == 200:
                dat = json.loads(resp.content.decode('utf-8'))
                if dat['deleted'] == 'yes':
                    logger.info('[OK] Deleted Scan: %s', md5)
                else:
                    logger.error('Deleting Scan: %s', md5)
                    return True
            else:
                logger.error('Deleting Scan: %s', md5)
                return True
        logger.info('Delete Scan Results API test completed')
    except Exception:
        logger.exception('Completing REST API Unit Test')
        return True
    return False


def start_test(request):
    """Static Analyzer Unit test."""
    item = request.GET.get('module', 'static')
    if item == 'static':
        comp = 'static_analyzer'
        failed_stat = static_analysis_test()
    else:
        comp = 'static_analyzer_api'
        failed_stat = api_test()
    try:
        if failed_stat:
            message = 'some tests failed'
            resp_code = 403
        else:
            message = 'all tests completed'
            resp_code = 200
    except Exception:
        resp_code = 403
        message = 'error'
    logger.info('\n\nALL TESTS COMPLETED!')
    logger.info('Test Status: %s', message)
    return HttpResponse(json.dumps({comp: message}),
                        content_type='application/json; charset=utf-8',
                        status=resp_code)


class StaticAnalyzerAndAPI(TestCase):
    """Unit Tests."""

    def setUp(self):
        self.http_client = Client()

    def test_static_analyzer(self):
        resp = self.http_client.post('/tests/?module=static')
        self.assertEqual(resp.status_code, 200)

    def test_rest_api(self):
        resp = self.http_client.post('/tests/?module=api')
        self.assertEqual(resp.status_code, 200)



# -*- coding: utf_8 -*-
"""Module for iOS IPA Binary Analysis."""

import logging
from pathlib import Path


from macholib.mach_o import (CPU_TYPE_NAMES, MH_CIGAM_64, MH_MAGIC_64,
                             get_cpu_subtype)
from macholib.MachO import MachO

from mobsf.StaticAnalyzer.views.ios.classdump import (
    get_class_dump,
)
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    MachOChecksec,
)
from mobsf.StaticAnalyzer.views.common.binary.strings import (
    strings_on_binary,
)
from mobsf.StaticAnalyzer.views.ios.binary_rule_matcher import (
    binary_rule_matcher,
)
from mobsf.MobSF.utils import (
    append_scan_status,
)

logger = logging.getLogger(__name__)


def detect_bin_type(libs):
    """Detect IPA binary type."""
    if any('libswiftCore.dylib' in itm for itm in libs):
        return 'Swift'
    else:
        return 'Objective C'


def get_bin_info(bin_file):
    """Get Binary Information."""
    logger.info('Getting Binary Information')
    m = MachO(bin_file.as_posix())
    for header in m.headers:
        if header.MH_MAGIC == MH_MAGIC_64 or header.MH_MAGIC == MH_CIGAM_64:
            sz = '64-bit'
        else:
            sz = '32-bit'
        arch = CPU_TYPE_NAMES.get(
            header.header.cputype, header.header.cputype)
        subarch = get_cpu_subtype(
            header.header.cputype, header.header.cpusubtype)
        return {'endian': header.endian,
                'bit': sz,
                'arch': arch,
                'subarch': subarch}


def ipa_macho_analysis(binary):
    data = {
        'checksec': {},
        'symbols': [],
        'libraries': [],
    }
    try:
        logger.info('Running MachO Analysis on: %s', binary.name)
        cs = MachOChecksec(binary)
        chksec = cs.checksec()
        symbols = cs.get_symbols()
        libs = cs.get_libraries()
        data['checksec'] = chksec
        data['symbols'] = symbols
        data['libraries'] = libs
    except Exception:
        logger.exception('Running MachO Analysis')
    return data


def binary_analysis(checksum, src, tools_dir, app_dir, executable_name):
    """Binary Analysis of IPA."""
    bin_dict = {
        'checksec': {},
        'libraries': [],
        'bin_code_analysis': {},
        'strings': [],
        'bin_info': {},
        'bin_type': '',
        'bin_path': None,
    }
    try:
        binary_findings = {}
        msg = 'Starting Binary Analysis'
        logger.info(msg)
        append_scan_status(checksum, msg)

        dot_app_path = next(Path(src).glob('**/*.app'), None)

        if not dot_app_path:
            logger.warning('Could not find .app directory.')
            return bin_dict

        bin_dir = dot_app_path

        if not executable_name:
            bin_name = dot_app_path.stem
        else:
            _bin = bin_dir / executable_name
            if _bin.exists():
                bin_name = executable_name
            else:
                bin_name = dot_app_path.stem

        bin_path = bin_dir / bin_name

        if not (bin_path.exists() and bin_path.is_file()):
            msg = (
                f'MobSF Cannot find binary in {bin_path.as_posix()}. '
                'Skipping Binary Analysis.')
            logger.warning(msg)
            append_scan_status(checksum, 'Skipping binary analysis', msg)
        else:
            macho = ipa_macho_analysis(bin_path)
            bin_info = get_bin_info(bin_path)
            bin_type = detect_bin_type(macho['libraries'])
            classdump = get_class_dump(
                checksum,
                tools_dir,
                bin_path,
                app_dir,
                bin_type)
            binary_rule_matcher(
                checksum,
                binary_findings,
                macho['symbols'], classdump)
            bin_dict['checksec'] = macho['checksec']
            bin_dict['libraries'] = macho['libraries']
            bin_dict['bin_code_analysis'] = binary_findings
            bin_dict['bin_info'] = bin_info
            bin_dict['bin_type'] = bin_type
            logger.info('Running strings against the Binary')
            bin_dict['strings'] = strings_on_binary(bin_path.as_posix())
            bin_dict['bin_path'] = bin_path
    except Exception as exp:
        msg = 'Failed to run IPA Binary Analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return bin_dict



# -*- coding: utf_8 -*-
"""Corellium APIs."""
import logging
import os
from copy import deepcopy
from socket import gethostname

from django.conf import settings

import requests

from mobsf.MobSF.utils import (
    is_number,
    upstream_proxy,
)


SUCCESS_RESP = (200, 204)
ERROR_RESP = (400, 403, 404, 409)
OK = 'ok'
CORELLIUM_DEFAULT = 'https://app.corellium.com'
CORELLIUM_API_DOMAIN = getattr(
    settings,
    'CORELLIUM_API_DOMAIN',
    CORELLIUM_DEFAULT)
if not CORELLIUM_API_DOMAIN:
    CORELLIUM_API_DOMAIN = CORELLIUM_DEFAULT
CORELLIUM_API_KEY = getattr(
    settings,
    'CORELLIUM_API_KEY', '')
logger = logging.getLogger(__name__)
TIMEOUT = 20


class CorelliumInit:

    def __init__(self) -> None:
        self.api = f'{CORELLIUM_API_DOMAIN}/api/v1'
        self.api_key = CORELLIUM_API_KEY
        self.headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.api_key}',
        }
        self.proxies, self.verify = upstream_proxy('https')


class CorelliumAPI(CorelliumInit):

    def __init__(self, project_id) -> None:
        super().__init__()
        self.project_id = project_id

    def api_ready(self):
        """Check API Availability."""
        try:
            r = requests.get(f'{self.api}/ready',
                             timeout=TIMEOUT,
                             proxies=self.proxies,
                             verify=self.verify)
            if r.status_code in SUCCESS_RESP:
                return True
            else:
                logger.error('Corellium API is not ready.'
                             ' Status code: %s', r.status_code)
        except Exception:
            logger.error('Network unreachable.')
            return False
        return False

    def api_auth(self):
        """Check Corellium API Auth."""
        if not self.api_key:
            logger.error('Corellium API key is not set')
            return False
        r = requests.get(
            f'{self.api}/projects',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in ERROR_RESP:
            return False
        return True

    def get_projects(self):
        """Get Projects."""
        logger.info('Getting Corellium project id')
        if self.project_id:
            return True
        else:
            ids = []
            r = requests.get(
                f'{self.api}/projects?ids_only=true',
                timeout=TIMEOUT,
                headers=self.headers,
                proxies=self.proxies,
                verify=self.verify)
            if r.status_code in SUCCESS_RESP:
                for i in r.json():
                    ids.append(i['id'])
            if ids:
                self.project_id = ids[0]
                return True
        return False

    def get_authorized_keys(self):
        """Get SSH public keys associated with a project."""
        r = requests.get(
            f'{self.api}/projects/{self.project_id}/keys',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()
        return False

    def add_authorized_key(self, key):
        """Add SSH public key to the Project."""
        logger.info('Adding SSH public key to Corellium project')
        extras = ''
        if os.getenv('MOBSF_PLATFORM') == 'docker':
            extras = ' - (docker)'
        data = {
            'kind': 'ssh',
            'label': f'MobSF SSH Key - {gethostname()}{extras}',
            'key': key,
        }
        r = requests.post(
            f'{self.api}/projects/{self.project_id}/keys',
            timeout=TIMEOUT,
            headers=self.headers,
            json=data,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()['identifier']
        return False

    def delete_authorized_key(self, key_id):
        """Delete SSH public key from the Project."""
        r = requests.delete(
            f'{self.api}/projects/{self.project_id}/keys/{key_id}',
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        return False

    def get_instances(self):
        """Get Instances."""
        logger.info('Getting iOS instances')
        instances = []
        r = requests.get(
            f'{self.api}/instances',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            for i in r.json():
                if i['type'] == 'ios' and 'jailbroken' in i['patches']:
                    instances.append(i)
        return instances

    def create_ios_instance(self, name, flavor, version):
        """Create a Jailbroken iOS instance."""
        data = {
            'project': self.project_id,
            'name': f'{name} - {flavor.upper()}',
            'flavor': flavor,
            'os': version,
        }
        r = requests.post(
            f'{self.api}/instances',
            timeout=TIMEOUT,
            headers=self.headers,
            json=data,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()['id']
        return False


class CorelliumModelsAPI(CorelliumInit):

    def get_models(self):
        r = requests.get(
            f'{self.api}/models',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()
        return False

    def get_supported_os(self, model):
        models = self.get_models()
        if not models:
            return False
        allowed = False
        for i in models:
            if i['type'] == 'ios' and model == i['model']:
                allowed = True
                break
        if not allowed:
            return False
        r = requests.get(
            f'{self.api}/models/{model}/software',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()
        elif r.status_code in ERROR_RESP:
            return r.json()['error']
        return False


class CorelliumInstanceAPI(CorelliumInit):

    def __init__(self, instance_id) -> None:
        super().__init__()
        self.instance_id = instance_id

    def start_instance(self):
        """Start instance."""
        data = {'paused': False}
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/start',
            timeout=TIMEOUT,
            headers=self.headers,
            json=data,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        elif r.status_code in ERROR_RESP:
            return r.json()['error']
        return False

    def stop_instance(self):
        """Stop instance."""
        data = {'soft': True}
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/stop',
            timeout=TIMEOUT,
            headers=self.headers,
            json=data,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        elif r.status_code in ERROR_RESP:
            return r.json()['error']
        return False

    def unpause_instance(self):
        """Unpause instance."""
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/unpause',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        elif r.status_code in ERROR_RESP:
            return r.json()['error']
        return False

    def reboot_instance(self):
        """Reboot instance."""
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/reboot',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        elif r.status_code in ERROR_RESP:
            return r.json()['error']
        return False

    def remove_instance(self):
        """Remove instance."""
        r = requests.delete(
            f'{self.api}/instances/{self.instance_id}',
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        elif r.status_code in ERROR_RESP:
            return r.json()['error']
        return False

    def poll_instance(self):
        """Check instance status."""
        r = requests.get(
            f'{self.api}/instances/{self.instance_id}',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()
        return False

    def screenshot(self):
        """Take screenshot inside VM."""
        r = None
        err = ''
        try:
            r = requests.get(
                (f'{self.api}/instances/{self.instance_id}'
                 '/screenshot.png?scale=1'),
                timeout=TIMEOUT,
                headers=self.headers,
                stream=True,
                proxies=self.proxies,
                verify=self.verify)
            if r.status_code == 200:
                return r.content
        except Exception:
            if r and r.json().get('error'):
                err = r.json()['error']
            logger.error('Failed to take a screenshot. %s', err)
        return False

    def start_network_capture(self):
        """Start network capture."""
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/sslsplit/enable',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        err = r.json()['error']
        if 'network monitoring enabled' in err:
            return OK
        logger.error(
            'Failed to enable network monitoring. %s', err)
        return r.json()['error']

    def stop_network_capture(self):
        """Stop network capture."""
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/sslsplit/disable',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        logger.error(
            'Failed to disable network monitoring. %s', r.json()['error'])
        return r.json()['error']

    def download_network_capture(self):
        """Download network capture."""
        r = requests.get(
            f'{self.api}/instances/{self.instance_id}/networkMonitor.pcap',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.content
        logger.error(
            'Failed to download pcap. %s', r.json()['error'])
        return None

    def console_log(self):
        """Get Console Log."""
        r = requests.get(
            f'{self.api}/instances/{self.instance_id}/consoleLog',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.content.decode('utf-8', 'ignore')
        logger.error(
            'Failed to disable network monitoring. %s', r.json()['error'])
        return r.json()['error']

    def get_ssh_connection_string(self):
        """Get SSH connection string."""
        r = requests.get(
            f'{self.api}/instances/{self.instance_id}/quickConnectCommand',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.text
        logger.error(
            'Failed to get SSH connection string %s', r.json()['error'])
        return r.json()['error']

    def device_input(self, event, x, y, max_x, max_y):
        """Provide touch/button event to VM."""
        if event == 'text':
            data = [{
                'text': x,
            }]
        else:
            if (not is_number(x)
                    or not is_number(y)
                    or not is_number(max_x)
                    or not is_number(max_y)):
                return 'Invalid coordinates'
            # Should not be greater than max screen size
            swipe_x = min(int(x) + 200, int(max_x))
            swipe_y = min(int(y) + 400, int(max_y))
            # Just to prevent linter complaining
            # Make use of these variables in future
            swipe_x = swipe_y
            swipe_y = swipe_x
            if event == 'home':
                data = [{
                    'buttons': ['holdButton'],
                }]
            elif event == 'enter':
                data = [{
                    'buttons': ['enter'],
                }]
            elif event == 'backspace':
                data = [{
                    'buttons': ['backspace'],
                }]
            elif event == 'left':
                data = [{
                    'buttons': ['left'],
                }]
            elif event == 'right':
                data = [{
                    'buttons': ['right'],
                }]
            elif event == 'swipe_up':
                data = [{
                    'startButtons': ['finger'],
                    'start': [[x, 600]],
                    'end': [[400, y]],
                    'duration': 200,
                }]
            elif event == 'swipe_down':
                data = [{
                    'startButtons': ['finger'],
                    'start': [[300, 600]],
                    'bezierPoints': [[[700, 350]], [[850, 375]]],
                    'end': [[950, 400]],
                    'duration': 200,
                }]
            elif event == 'swipe_left':
                data = [{
                    'startButtons': ['finger'],
                    'start': [[100, 900]],
                    'bezierPoints': [[[700, 350]], [[850, 375]]],
                    'end': [[900, 100]],
                    'duration': 200,
                }]
            elif event == 'swipe_right':
                data = [{
                    'startButtons': ['finger'],
                    'start': [[700, 100]],
                    'bezierPoints': [[[350, 750]], [[375, 875]]],
                    'end': [[100, 700]],
                    'duration': 200,
                }]
            else:
                data = [
                    {'buttons': ['finger'],
                     'position': [[x, y]],
                     'wait': 0},
                    {'buttons': [], 'wait': 100}]
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/input',
            timeout=TIMEOUT,
            headers=self.headers,
            json=data,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return OK
        logger.error(
            'Failed to send touch event. %s', r.json()['error'])
        return r.json()['error']


class CorelliumAgentAPI(CorelliumInit):

    def __init__(self, instance_id) -> None:
        super().__init__()
        self.instance_id = instance_id

    def agent_ready(self):
        """Agent ready."""
        r = requests.get(
            f'{self.api}/instances/{self.instance_id}/agent/v1/app/ready',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('Corellium Agent is Ready!')
            return r.json()['ready']
        elif r.status_code in ERROR_RESP:
            logger.error('Corellium Agent is not Ready, Please Wait!')
            return r.json()['error']
        return False

    def unlock_device(self):
        """Unlock iOS device."""
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/agent/v1/system/unlock',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('Device unlocked')
            return OK
        elif r.status_code in ERROR_RESP:
            logger.error('Failed to unlock device')
            return r.json()['error']
        return False

    def upload_ipa(self, ipa_file):
        """Upload IPA."""
        logger.info('Uploading IPA to iOS instance...')
        headers = deepcopy(self.headers)
        headers['Content-Type'] = 'application/octet-stream'
        r = requests.put(
            (f'{self.api}/instances/{self.instance_id}'
             f'/agent/v1/file/device/%2Ftmp%2Fapp.ipa'),
            data=open(ipa_file, 'rb').read(),
            headers=headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('IPA uploaded to instance')
            return OK
        logger.error('Failed to upload IPA %s', r.json()['error'])
        return r.json()['error']

    def install_ipa(self):
        """Install IPA."""
        r = requests.post(
            f'{self.api}/instances/{self.instance_id}/agent/v1/app/install',
            timeout=TIMEOUT,
            headers=self.headers,
            json={'path': '/tmp/app.ipa'},
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('App installed')
            return OK
        logger.error('Failed to install the IPA. %s', r.json()['error'])
        return r.json()['error']

    def run_app(self, bundle_id):
        """Run an App."""
        r = requests.post(
            (f'{self.api}/instances/{self.instance_id}'
             f'/agent/v1/app/apps/{bundle_id}/run'),
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('App Started')
            return OK
        logger.error('Failed to start the app. %s', r.json()['error'])
        return r.json()['error']

    def stop_app(self, bundle_id):
        """Stop an App."""
        r = requests.post(
            (f'{self.api}/instances/{self.instance_id}'
             f'/agent/v1/app/apps/{bundle_id}/kill'),
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('App Killed')
            return OK
        logger.error('Failed to stop the app. %s', r.json()['error'])
        return r.json()['error']

    def remove_app(self, bundle_id):
        """Remove an app from VM."""
        r = requests.post(
            (f'{self.api}/instances/{self.instance_id}'
             f'/agent/v1/app/apps/{bundle_id}/uninstall'),
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            logger.info('App Removed')
            return OK
        logger.error('Failed to remove the app. %s', r.json()['error'])
        return r.json()['error']

    def list_apps(self):
        """List all apps installed."""
        r = requests.get(
            f'{self.api}/instances/{self.instance_id}/agent/v1/app/apps',
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()
        elif r.status_code in ERROR_RESP:
            return r.json()
        return False

    def get_icons(self, bundleids):
        """Get app icons by bundleId."""
        r = requests.get(
            (f'{self.api}/instances/{self.instance_id}'
             f'/agent/v1/app/icons?{bundleids}'),
            timeout=TIMEOUT,
            headers=self.headers,
            proxies=self.proxies,
            verify=self.verify)
        if r.status_code in SUCCESS_RESP:
            return r.json()
        elif r.status_code in ERROR_RESP:
            return r.json()
        return False



import logging

import requests

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    get_config_loc,
    upstream_proxy,
)

logger = logging.getLogger(__name__)


class VirusTotal:

    API_KEY_ERROR = 'VirusTotal Permission denied, wrong api key?'
    API_KEY_ERROR_SHORT = 'VirusTotal API error'
    API_CONN_ERROR = 'VirusTotal Connection Error'

    def __init__(self, checksum):
        self.base_url = settings.VIRUS_TOTAL_BASE_URL
        self.checksum = checksum

    def get_report(self):
        """
        Get Report from VT.

        :param file_hash: md5/sha1/sha256
        :return: json response / None
        """
        try:
            file_hash = self.checksum
            url = self.base_url + 'report'
            params = {
                'apikey': settings.VT_API_KEY,
                'resource': file_hash}
            headers = {'Accept-Encoding': 'gzip, deflate'}
            try:
                proxies, verify = upstream_proxy('https')
            except Exception as exp:
                msg = 'Setting upstream proxy'
                logger.exception(msg)
                append_scan_status(file_hash, msg, repr(exp))
            try:
                response = requests.get(
                    url,
                    timeout=5,
                    params=params,
                    headers=headers,
                    proxies=proxies,
                    verify=verify)
                if response.status_code == 403:
                    logger.error(self.API_KEY_ERROR)
                    append_scan_status(
                        file_hash,
                        self.API_KEY_ERROR,
                        self.API_KEY_ERROR_SHORT)
                    return None
            except Exception as exp:
                logger.error(self.API_CONN_ERROR)
                append_scan_status(
                    file_hash,
                    self.API_CONN_ERROR,
                    repr(exp))
                return None
            try:
                json_response = response.json()
                return json_response
            except ValueError:
                return None
        except Exception as exp:
            msg = 'Failed to get report from VirusTotal'
            logger.exception(msg)
            append_scan_status(file_hash, msg, repr(exp))
            return None

    def upload_file(self, file_path):
        """
        Upload File to VT.

        :param file_path: file path to upload
        :return: json response / None
        """
        try:
            url = self.base_url + 'scan'
            if file_size(file_path) > 31:
                logger.warning('VirusTotal Public API does '
                               'not support files above 32 MB')
                return None
            files = {'file': open(file_path, 'rb')}
            headers = {'apikey': settings.VT_API_KEY}
            try:
                proxies, verify = upstream_proxy('https')
            except Exception as exp:
                msg = 'Setting upstream proxy'
                logger.exception(msg)
                append_scan_status(self.checksum, msg, repr(exp))
            try:
                response = requests.post(
                    url,
                    timeout=5,
                    files=files,
                    data=headers,
                    proxies=proxies,
                    verify=verify)
                if response.status_code == 403:
                    logger.error(self.API_KEY_ERROR)
                    append_scan_status(
                        self.checksum,
                        self.API_KEY_ERROR,
                        self.API_KEY_ERROR_SHORT)
                    return None
            except Exception as exp:
                logger.error(self.API_CONN_ERROR)
                append_scan_status(
                    self.checksum,
                    self.API_CONN_ERROR,
                    repr(exp))
                return None
            json_response = response.json()
            return json_response

        except Exception as exp:
            msg = 'Failed to upload file to VirusTotal'
            logger.exception(msg)
            append_scan_status(self.checksum, msg, repr(exp))
            return None

    def get_result(self, file_path):
        """
        Get Results from VT.

        Uploading a file and getting the approval msg from VT
        or fetching existing report
        :param file_path: file's path
        :return: VirusTotal result json / None upon error
        """
        try:
            file_hash = self.checksum
            msg = 'VirusTotal: Check for existing report'
            logger.info(msg)
            append_scan_status(file_hash, msg)
            report = self.get_report()
            # Check for existing report
            if report:
                if report['response_code'] == 1:
                    msg = f'VirusTotal: {report["verbose_msg"]}'
                    logger.info(msg)
                    append_scan_status(file_hash, msg)
                    return report
            if settings.VT_UPLOAD:
                msg = 'VirusTotal: file upload'
                logger.info(msg)
                append_scan_status(file_hash, msg)
                upload_response = self.upload_file(file_path)
                if upload_response:
                    msg = f'VirusTotal: {upload_response["verbose_msg"]}'
                    logger.info(msg)
                    append_scan_status(file_hash, msg)
                return upload_response
            else:
                msg = ('VirusTotal Scan not performed as file '
                       f'upload is disabled in {get_config_loc()}. '
                       'To enable file upload, set VT_UPLOAD to True.')
                logger.info(msg)
                append_scan_status(file_hash, msg)
                message = ('Scan not performed, VirusTotal file '
                           f'upload disabled in {get_config_loc()}')
                report = {
                    'verbose_msg': message,
                    'positives': 0,
                    'total': 0}
                return report
        except Exception as exp:
            msg = 'VirusTotal: Error getting result'
            logger.exception(msg)
            append_scan_status(file_hash, msg, repr(exp))



# -*- coding: utf_8 -*-
# flake8: noqa
"""Module for android manifest analysis."""
import logging
from urllib.parse import urlparse

import requests
from concurrent.futures import ThreadPoolExecutor
from django.utils.html import escape

from mobsf.MobSF.utils import (
    append_scan_status,
    is_number,
    upstream_proxy,
)
from mobsf.MobSF.security import valid_host
from mobsf.StaticAnalyzer.views.android import (
    network_security,
)
from mobsf.StaticAnalyzer.views.android.kb import (
    android_manifest_desc,
)


logger = logging.getLogger(__name__)
ANDROID_4_2_LEVEL = 17
ANDROID_5_0_LEVEL = 21
ANDROID_8_0_LEVEL = 26
ANDROID_9_0_LEVEL = 28
ANDROID_10_0_LEVEL = 29
ANDROID_MANIFEST_FILE = 'AndroidManifest.xml'
WELL_KNOWN_PATH = '/.well-known/assetlinks.json'

ANDROID_API_LEVEL_MAP = {
    '1': '1.0',
    '2': '1.1',
    '3': '1.5',
    '4': '1.6',
    '5': '2.0-2.1',
    '8': '2.2-2.2.3',
    '9': '2.3-2.3.2',
    '10': '2.3.3-2.3.7',
    '11': '3.0',
    '12': '3.1',
    '13': '3.2-3.2.6',
    '14': '4.0-4.0.2',
    '15': '4.0.3-4.0.4',
    '16': '4.1-4.1.2',
    '17': '4.2-4.2.2',
    '18': '4.3-4.3.1',
    '19': '4.4-4.4.4',
    '20': '4.4W-4.4W.2',
    '21': '5.0-5.0.2',
    '22': '5.1-5.1.1',
    '23': '6.0-6.0.1',
    '24': '7.0',
    '25': '7.1-7.1.2',
    '26': '8.0',
    '27': '8.1',
    '28': '9',
    '29': '10',
    '30': '11',
    '31': '12',
    '32': '12L',
    '33': '13',
    '34': '14',
    '35': '15',
    '36': '16',
}


def escape_manifest_attribute(value):
    """Escape manifest attributes."""
    if not value:
        return value
    return escape(value)


def assetlinks_check(act_name, well_knowns):
    """Well known assetlink check."""
    findings = []

    with ThreadPoolExecutor() as executor:
        futures = []
        for w_url, host in well_knowns.items():
            logger.info(
                'App Link Assetlinks Check - [%s] %s', act_name, host)
            futures.append(
                executor.submit(_check_url, host, w_url))
        for future in futures:
            findings.append(future.result())

    return findings


def _check_url(host, w_url):
    """Check for the presence of Assetlinks URL."""
    try:
        iden = 'sha256_cert_fingerprints'
        proxies, verify = upstream_proxy('https')
        status = False
        status_code = 0

        urls = {w_url}
        if w_url.startswith('http://'):
            # Upgrade http to https
            urls.add(f'https://{w_url[7:]}')

        for url in urls:
            # Validate the fully-assembled URL — path, port, query, and params.
            purl = urlparse(url)
            if (purl.path != WELL_KNOWN_PATH
                or len(purl.query) > 0
                    or len(purl.params) > 0):
                logger.warning('Invalid Assetlinks URL: %s', url)
                continue
            if purl.port and purl.port not in (80, 443):
                logger.warning(
                    'Non-standard port in assetlinks URL rejected: %s', url)
                continue
            r = requests.get(url,
                             timeout=5,
                             allow_redirects=False,
                             proxies=proxies,
                             verify=verify)

            status_code = r.status_code
            if (str(status_code).startswith('2') and iden in str(r.json())):
                status = True
                break
        if status_code in (301, 302):
            logger.warning('Status Code: [%d], Redirecting to '
                           'a different URL, skipping check!', status_code)
        return {'url': w_url,
                'host': host,
                'status_code': status_code,
                'status': status}

    except Exception:
        logger.error(f'Well Known Assetlinks Check for URL: {w_url}')
        return {'url': w_url,
                'host': host,
                'status_code': None,
                'status': False}


def get_browsable_activities(node, ns):
    """Get Browsable Activities."""
    try:
        browse_dic = {}
        schemes = []
        mime_types = []
        hosts = []
        ports = []
        paths = []
        path_prefixs = []
        path_patterns = []
        well_known = {}
        catg = node.getElementsByTagName('category')
        for cat in catg:
            if cat.getAttribute(f'{ns}:name') == 'android.intent.category.BROWSABLE':
                data_tag = node.getElementsByTagName('data')
                for data in data_tag:
                    scheme = data.getAttribute(f'{ns}:scheme')
                    if scheme and scheme not in schemes:
                        schemes.append(scheme)
                    mime = data.getAttribute(f'{ns}:mimeType')
                    if mime and mime not in mime_types:
                        mime_types.append(mime)
                    host = data.getAttribute(f'{ns}:host')
                    if host and host not in hosts:
                        hosts.append(host)
                    port = data.getAttribute(f'{ns}:port')
                    if port and port not in ports:
                        ports.append(port)
                    path = data.getAttribute(f'{ns}:path')
                    if path and path not in paths:
                        paths.append(path)
                    path_prefix = data.getAttribute(f'{ns}:pathPrefix')
                    if path_prefix and path_prefix not in path_prefixs:
                        path_prefixs.append(path_prefix)
                    path_pattern = data.getAttribute(f'{ns}:pathPattern')
                    if path_pattern and path_pattern not in path_patterns:
                        path_patterns.append(path_pattern)
                    # Collect possible well-known paths
                    if (scheme
                        and scheme in ('http', 'https')
                        and host
                            and host != '*'):
                        host = host.replace('*.', '').replace('#', '')
                        if not valid_host(host):
                            logger.warning('Invalid Host: %s', host)
                            continue
                        shost = f'{scheme}://{host}'
                        if port and is_number(port):
                            if int(port) not in (80, 443):
                                logger.warning(
                                    'Non-standard port rejected in assetlinks '
                                    'check (port %s bypasses valid_host): %s',
                                    port, host)
                                continue
                            c_url = f'{shost}:{port}{WELL_KNOWN_PATH}'
                        else:
                            c_url = f'{shost}{WELL_KNOWN_PATH}'
                        well_known[c_url] = shost
        schemes = [scheme + '://' for scheme in schemes]
        browse_dic['schemes'] = schemes
        browse_dic['mime_types'] = mime_types
        browse_dic['hosts'] = hosts
        browse_dic['ports'] = ports
        browse_dic['paths'] = paths
        browse_dic['path_prefixs'] = path_prefixs
        browse_dic['path_patterns'] = path_patterns
        browse_dic['browsable'] = bool(browse_dic['schemes'])
        browse_dic['well_known'] = well_known
        return browse_dic
    except Exception:
        logger.exception('Getting Browsable Activities')


def manifest_analysis(app_dic, man_data_dic):
    """Analyse manifest file."""
    # pylint: disable=C0301
    checksum = app_dic['md5']
    mfxml = app_dic['manifest_parsed_xml']
    ns = app_dic['manifest_namespace']
    src_type = app_dic['zipped']
    app_dir = app_dic['app_dir']
    try:
        msg = 'Manifest Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        exp_count = dict.fromkeys(['act', 'ser', 'bro', 'cnt'], 0)
        applications = mfxml.getElementsByTagName('application')
        data_tag = mfxml.getElementsByTagName('data')
        intents = mfxml.getElementsByTagName('intent-filter')
        actions = mfxml.getElementsByTagName('action')
        granturipermissions = mfxml.getElementsByTagName(
            'grant-uri-permission')
        permissions = mfxml.getElementsByTagName('permission')
        ret_value = []
        ret_list = []
        exported = []
        browsable_activities = {}
        permission_dict = {}
        do_netsec = False
        debuggable = False
        # PERMISSION
        for permission in permissions:
            if permission.getAttribute(f'{ns}:protectionLevel'):
                protectionlevel = permission.getAttribute(
                    f'{ns}:protectionLevel')
                if protectionlevel == '0x00000000':
                    protectionlevel = 'normal'
                elif protectionlevel == '0x00000001':
                    protectionlevel = 'dangerous'
                elif protectionlevel == '0x00000002':
                    protectionlevel = 'signature'
                elif protectionlevel == '0x00000003':
                    protectionlevel = 'signatureOrSystem'

                permission_dict[permission.getAttribute(
                    f'{ns}:name')] = protectionlevel
            elif permission.getAttribute(f'{ns}:name'):
                permission_dict[permission.getAttribute(
                    f'{ns}:name')] = 'normal'
        # GENERAL
        if man_data_dic['min_sdk'] and int(man_data_dic['min_sdk']) < ANDROID_8_0_LEVEL:
            minsdk = man_data_dic.get('min_sdk')
            android_version = ANDROID_API_LEVEL_MAP.get(minsdk, 'XX')
            ret_list.append(('vulnerable_os_version', (android_version, minsdk,), ()))
        elif man_data_dic['min_sdk'] and int(man_data_dic['min_sdk']) < ANDROID_10_0_LEVEL:
            minsdk = man_data_dic.get('min_sdk')
            android_version = ANDROID_API_LEVEL_MAP.get(minsdk, 'XX')
            ret_list.append(('vulnerable_os_version2', (android_version, minsdk,), ()))
        # APPLICATIONS
        # Handle multiple application tags in AAR
        backupDisabled = False
        for application in applications:
            # Esteve 23.07.2016 - begin - identify permission at the
            # application level
            if application.getAttribute(f'{ns}:permission'):
                perm_appl_level_exists = True
                perm_appl_level = application.getAttribute(
                    f'{ns}:permission')
            else:
                perm_appl_level_exists = False
            # End
            if application.getAttribute(f'{ns}:usesCleartextTraffic') == 'true':
                ret_list.append(('clear_text_traffic', (), ()))
            if application.getAttribute(f'{ns}:directBootAware') == 'true':
                ret_list.append(('direct_boot_aware', (), ()))
            if application.getAttribute(f'{ns}:networkSecurityConfig'):
                item = application.getAttribute(f'{ns}:networkSecurityConfig')
                ret_list.append(('has_network_security', (item,), ()))
                do_netsec = item
            if application.getAttribute(f'{ns}:debuggable') == 'true':
                ret_list.append(('app_is_debuggable', (), ()))
                debuggable = True
            if application.getAttribute(f'{ns}:allowBackup') == 'true':
                ret_list.append(('app_allowbackup', (), ()))
            elif application.getAttribute(f'{ns}:allowBackup') == 'false':
                backupDisabled = True
            else:
                if not backupDisabled:
                    ret_list.append(('allowbackup_not_set', (), ()))
            if application.getAttribute(f'{ns}:testOnly') == 'true':
                ret_list.append(('app_in_test_mode', (), ()))
            for node in application.childNodes:
                an_or_a = ''
                if node.nodeName == 'activity':
                    itemname = 'Activity'
                    cnt_id = 'act'
                    an_or_a = 'n'
                elif node.nodeName == 'activity-alias':
                    itemname = 'Activity-Alias'
                    cnt_id = 'act'
                    an_or_a = 'n'
                elif node.nodeName == 'provider':
                    itemname = 'Content Provider'
                    cnt_id = 'cnt'
                elif node.nodeName == 'receiver':
                    itemname = 'Broadcast Receiver'
                    cnt_id = 'bro'
                elif node.nodeName == 'service':
                    itemname = 'Service'
                    cnt_id = 'ser'
                else:
                    itemname = 'NIL'
                item = ''
                # Checks for Activities
                if itemname in ['Activity', 'Activity-Alias']:
                    item = node.getAttribute(f'{ns}:name')
                    item = escape_manifest_attribute(item)
                    # Browsable Activities
                    browse_dic = get_browsable_activities(node, ns)
                    if browse_dic['browsable']:
                        browsable_activities[node.getAttribute(
                            f'{ns}:name')] = browse_dic
                    for finding in assetlinks_check(item, browse_dic['well_known']):
                        if not finding['status']:
                            ret_list.append(('well_known_assetlinks',
                                            (item, finding['host']),
                                            (finding['url'],
                                             finding['status_code'])))

                    # Task Affinity
                    task_affinity = node.getAttribute(f'{ns}:taskAffinity')
                    if (task_affinity):
                        ret_list.append(('task_affinity_set', (item,), ()))

                    # LaunchMode
                    try:
                        affected_sdk = int(
                            man_data_dic['min_sdk']) < ANDROID_5_0_LEVEL
                    except Exception:
                        # in case min_sdk is not defined we assume vulnerability
                        affected_sdk = True
                    launchmode = node.getAttribute(f'{ns}:launchMode')
                    modes = ('singleTask', 'singleInstance')
                    if (affected_sdk
                            and launchmode in modes):
                        ret_list.append(('non_standard_launchmode', (item,), ()))

                    # Android Task Hijacking or StrandHogg 1.0
                    try:
                        target_sdk = int(man_data_dic['target_sdk'])
                    except Exception:
                        try:
                            target_sdk = int(man_data_dic['min_sdk'])
                        except Exception:
                            target_sdk = ANDROID_8_0_LEVEL
                    if (target_sdk < ANDROID_9_0_LEVEL
                            and launchmode == 'singleTask'):
                        ret_list.append(('task_hijacking', (item,), (target_sdk,)))

                    # Android StrandHogg 2.0
                    exported_act = node.getAttribute(f'{ns}:exported')
                    if (target_sdk < ANDROID_10_0_LEVEL
                            and exported_act == 'true'
                            and (launchmode != 'singleInstance' or task_affinity != '')):
                        ret_list.append(('task_hijacking2', (item,), (target_sdk,)))

                # Exported Check
                item = ''
                is_inf = False
                is_perm_exist = False
                # Esteve 23.07.2016 - begin - initialise variables to identify
                # the existence of a permission at the component level that
                # matches a permission at the manifest level
                prot_level_exist = False
                protlevel = ''
                # End
                if itemname != 'NIL':
                    if node.getAttribute(f'{ns}:exported') == 'true':
                        perm = ''
                        item = node.getAttribute(f'{ns}:name')
                        item = escape_manifest_attribute(item)
                        if node.getAttribute(f'{ns}:permission'):
                            # permission exists
                            perm = ('<strong>Permission: </strong>'
                                    + node.getAttribute(f'{ns}:permission'))
                            is_perm_exist = True
                        if item != man_data_dic['mainactivity']:
                            if is_perm_exist:
                                prot = ''
                                if node.getAttribute(f'{ns}:permission') in permission_dict:
                                    prot = ('</br><strong>protectionLevel: </strong>'
                                            + permission_dict[node.getAttribute(f'{ns}:permission')])
                                    # Esteve 23.07.2016 - begin - take into account protection level of the permission when claiming that a component is protected by it;
                                    # - the permission might not be defined in the application being analysed, if so, the protection level is not known;
                                    # - activities (or activity-alias) that are exported and have an unknown or normal or dangerous protection level are
                                    # included in the EXPORTED data structure for further treatment; components in this situation are also
                                    # counted as exported.
                                    prot_level_exist = True
                                    protlevel = permission_dict[
                                        node.getAttribute(f'{ns}:permission')]
                                if prot_level_exist:
                                    if protlevel == 'normal':
                                        ret_list.append(
                                            ('exported_protected_permission_normal', (itemname, item, perm + prot), (an_or_a, itemname)))
                                        if itemname in ['Activity', 'Activity-Alias']:
                                            exported.append(item)
                                        exp_count[cnt_id] = exp_count[
                                            cnt_id] + 1
                                    elif protlevel == 'dangerous':
                                        ret_list.append(
                                            ('exported_protected_permission_dangerous', (itemname, item, perm + prot), (an_or_a, itemname)))
                                        if itemname in ['Activity', 'Activity-Alias']:
                                            exported.append(item)
                                        exp_count[cnt_id] = exp_count[
                                            cnt_id] + 1
                                    elif protlevel == 'signature':
                                        ret_list.append(
                                            ('exported_protected_permission_signature', (itemname, item, perm + prot), (an_or_a, itemname)))
                                    elif protlevel == 'signatureOrSystem':
                                        ret_list.append(
                                            ('exported_protected_permission_signatureorsystem', (itemname, item, perm + prot), (an_or_a, itemname)))
                                else:
                                    ret_list.append(
                                        ('exported_protected_permission_not_defined', (itemname, item, perm), (an_or_a, itemname)))
                                    if itemname in ['Activity', 'Activity-Alias']:
                                        exported.append(item)
                                    exp_count[cnt_id] = exp_count[cnt_id] + 1
                                # Esteve 23.07.2016 - end
                            else:
                                # Esteve 24.07.2016 - begin - At this point, we are dealing with components that do not have a permission neither at the component level nor at the
                                # application level. As they are exported, they
                                # are not protected.
                                if perm_appl_level_exists is False:
                                    ret_list.append(
                                        ('explicitly_exported', (itemname, item), (an_or_a, itemname)))
                                    if itemname in ['Activity', 'Activity-Alias']:
                                        exported.append(item)
                                    exp_count[cnt_id] = exp_count[cnt_id] + 1
                                # Esteve 24.07.2016 - end
                                # Esteve 24.07.2016 - begin - At this point, we are dealing with components that have a permission at the application level, but not at the component
                                #  level. Two options are possible:
                                #        1) The permission is defined at the manifest level, which allows us to differentiate the level of protection as
                                #           we did just above for permissions specified at the component level.
                                #        2) The permission is not defined at the manifest level, which means the protection level is unknown, as it is not
                                # defined in the analysed application.
                                else:
                                    perm = '<strong>Permission: </strong>' + perm_appl_level
                                    prot = ''
                                    if perm_appl_level in permission_dict:
                                        prot = ('</br><strong>protectionLevel: </strong>'
                                                + permission_dict[perm_appl_level])
                                        prot_level_exist = True
                                        protlevel = permission_dict[
                                            perm_appl_level]
                                    if prot_level_exist:
                                        if protlevel == 'normal':
                                            ret_list.append(
                                                ('exported_protected_permission_normal_app_level', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            if itemname in ['Activity', 'Activity-Alias']:
                                                exported.append(item)
                                            exp_count[cnt_id] = exp_count[
                                                cnt_id] + 1
                                        elif protlevel == 'dangerous':
                                            ret_list.append(
                                                ('exported_protected_permission_dangerous_app_level', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            if itemname in ['Activity', 'Activity-Alias']:
                                                exported.append(item)
                                            exp_count[cnt_id] = exp_count[
                                                cnt_id] + 1
                                        elif protlevel == 'signature':
                                            ret_list.append(
                                                ('exported_protected_permission', (itemname, item, perm + prot), (an_or_a, itemname)))
                                        elif protlevel == 'signatureOrSystem':
                                            ret_list.append(
                                                ('exported_protected_permission_signatureorsystem_app_level', (itemname, item, perm + prot), (an_or_a, itemname)))
                                    else:
                                        ret_list.append(
                                            ('exported_protected_permission_app_level', (itemname, item, perm), (an_or_a, itemname)))
                                        if itemname in ['Activity', 'Activity-Alias']:
                                            exported.append(item)
                                        exp_count[cnt_id] = exp_count[
                                            cnt_id] + 1
                                # Esteve 24.07.2016 - end

                    elif node.getAttribute(f'{ns}:exported') != 'false':
                        # Check for Implicitly Exported
                        # Logic to support intent-filter
                        intentfilters = node.childNodes
                        for i in intentfilters:
                            inf = i.nodeName
                            if inf == 'intent-filter':
                                is_inf = True
                        if is_inf:
                            item = node.getAttribute(f'{ns}:name')
                            item = escape_manifest_attribute(item)
                            if node.getAttribute(f'{ns}:permission'):
                                # permission exists
                                perm = ('<strong>Permission: </strong>'
                                        + node.getAttribute(f'{ns}:permission'))
                                is_perm_exist = True
                            if item != man_data_dic['mainactivity']:
                                if is_perm_exist:
                                    prot = ''
                                    if node.getAttribute(f'{ns}:permission') in permission_dict:
                                        prot = ('</br><strong>protectionLevel: </strong>'
                                                + permission_dict[node.getAttribute(f'{ns}:permission')])
                                        # Esteve 24.07.2016 - begin - take into account protection level of the permission when claiming that a component is protected by it;
                                        # - the permission might not be defined in the application being analysed, if so, the protection level is not known;
                                        # - activities (or activity-alias) that are exported and have an unknown or normal or dangerous protection level are
                                        #  included in the EXPORTED data structure for further treatment; components in this situation are also
                                        #  counted as exported.
                                        prot_level_exist = True
                                        protlevel = permission_dict[
                                            node.getAttribute(f'{ns}:permission')]
                                        if prot_level_exist:
                                            if protlevel == 'normal':
                                                ret_list.append(
                                                    ('exported_protected_permission_normal', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                if itemname in ['Activity', 'Activity-Alias']:
                                                    exported.append(item)
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            elif protlevel == 'dangerous':
                                                ret_list.append(
                                                    ('exported_protected_permission_dangerous', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                if itemname in ['Activity', 'Activity-Alias']:
                                                    exported.append(item)
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            elif protlevel == 'signature':
                                                ret_list.append(
                                                    ('exported_protected_permission_signature', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            elif protlevel == 'signatureOrSystem':
                                                ret_list.append(
                                                    ('exported_protected_permission_signatureorsystem', (itemname, item, perm + prot), (an_or_a, itemname)))
                                    else:
                                        ret_list.append(
                                            ('exported_protected_permission_not_defined', (itemname, item, perm), (an_or_a, itemname)))
                                        if itemname in ['Activity', 'Activity-Alias']:
                                            exported.append(item)
                                        exp_count[cnt_id] = exp_count[
                                            cnt_id] + 1
                                    # Esteve 24.07.2016 - end
                                else:
                                    # Esteve 24.07.2016 - begin - At this point, we are dealing with components that do not have a permission neither at the component level nor at the
                                    # application level. As they are exported,
                                    # they are not protected.
                                    if perm_appl_level_exists is False:
                                        ret_list.append(
                                            ('exported_intent_filter_exists', (itemname, item), (an_or_a, itemname, itemname)))
                                        if itemname in ['Activity', 'Activity-Alias']:
                                            exported.append(item)
                                        exp_count[cnt_id] = exp_count[
                                            cnt_id] + 1
                                    # Esteve 24.07.2016 - end
                                    # Esteve 24.07.2016 - begin - At this point, we are dealing with components that have a permission at the application level, but not at the component
                                    # level. Two options are possible:
                                    # 1) The permission is defined at the manifest level, which allows us to differentiate the level of protection as
                                    #  we did just above for permissions specified at the component level.
                                    # 2) The permission is not defined at the manifest level, which means the protection level is unknown, as it is not
                                    #  defined in the analysed application.
                                    else:
                                        perm = '<strong>Permission: </strong>' + perm_appl_level
                                        prot = ''
                                        if perm_appl_level in permission_dict:
                                            prot = ('</br><strong>protectionLevel: </strong>'
                                                    + permission_dict[perm_appl_level])
                                            prot_level_exist = True
                                            protlevel = permission_dict[
                                                perm_appl_level]
                                        if prot_level_exist:
                                            if protlevel == 'normal':
                                                ret_list.append(
                                                    ('exported_protected_permission_normal_app_level', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                if itemname in ['Activity', 'Activity-Alias']:
                                                    exported.append(item)
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            elif protlevel == 'dangerous':
                                                ret_list.append(
                                                    ('exported_protected_permission_dangerous_app_level', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                if itemname in ['Activity', 'Activity-Alias']:
                                                    exported.append(item)
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            elif protlevel == 'signature':
                                                ret_list.append(
                                                    ('exported_protected_permission', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            elif protlevel == 'signatureOrSystem':
                                                ret_list.append(
                                                    ('exported_protected_permission_signatureorsystem_app_level', (itemname, item, perm + prot), (an_or_a, itemname)))
                                        else:
                                            ret_list.append(
                                                ('exported_protected_permission_app_level', (itemname, item, perm), (an_or_a, itemname)))
                                            if itemname in ['Activity', 'Activity-Alias']:
                                                exported.append(item)
                                            exp_count[cnt_id] = exp_count[
                                                cnt_id] + 1
                                    # Esteve 24.07.2016 - end
                                    # Esteve 29.07.2016 - begin The component is not explicitly exported (android:exported is not 'true'). It is not implicitly exported either (it does not
                                    # make use of an intent filter). Despite that, it could still be exported by default, if it is a content provider and the android:targetSdkVersion
                                    # is older than 17 (Jelly Bean, Android version 4.2). This is true regardless of the system's API level.
                                    # Finally, it must also be taken into account that, if the minSdkVersion is greater or equal than 17, this check is unnecessary, because the
                                    # app will not be run on a system where the
                                    # system's API level is below 17.
                        else:
                            if man_data_dic['min_sdk'] and man_data_dic['target_sdk'] and int(man_data_dic['min_sdk']) < ANDROID_4_2_LEVEL:
                                if itemname == 'Content Provider' and int(man_data_dic['target_sdk']) < ANDROID_4_2_LEVEL:
                                    perm = ''
                                    item = node.getAttribute(f'{ns}:name')
                                    item = escape_manifest_attribute(item)
                                    if node.getAttribute(f'{ns}:permission'):
                                        # permission exists
                                        perm = ('<strong>Permission: </strong>'
                                                + node.getAttribute(f'{ns}:permission'))
                                        is_perm_exist = True
                                    if is_perm_exist:
                                        prot = ''
                                        if node.getAttribute(f'{ns}:permission') in permission_dict:
                                            prot = ('</br><strong>protectionLevel: </strong>'
                                                    + permission_dict[node.getAttribute(f'{ns}:permission')])
                                            prot_level_exist = True
                                            protlevel = permission_dict[
                                                node.getAttribute(f'{ns}:permission')]
                                        if prot_level_exist:
                                            if protlevel == 'normal':
                                                ret_list.append(
                                                    ('exported_provider_normal', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            elif protlevel == 'dangerous':
                                                ret_list.append(
                                                    ('exported_provider_danger', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            elif protlevel == 'signature':
                                                ret_list.append(
                                                    ('exported_provider_signature', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            elif protlevel == 'signatureOrSystem':
                                                ret_list.append(
                                                    ('exported_provider_signatureorsystem', (itemname, item, perm + prot), (an_or_a, itemname)))
                                        else:
                                            ret_list.append(
                                                ('exported_provider_unknown', (itemname, item, perm), (an_or_a, itemname)))
                                            exp_count[cnt_id] = exp_count[
                                                cnt_id] + 1
                                    else:
                                        if perm_appl_level_exists is False:
                                            ret_list.append(
                                                ('exported_provider', (itemname, item), (an_or_a, itemname)))
                                            exp_count[cnt_id] = exp_count[
                                                cnt_id] + 1
                                        else:
                                            perm = '<strong>Permission: </strong>' + perm_appl_level
                                            prot = ''
                                            if perm_appl_level in permission_dict:
                                                prot = ('</br><strong>protectionLevel: </strong>'
                                                        + permission_dict[perm_appl_level])
                                                prot_level_exist = True
                                                protlevel = permission_dict[
                                                    perm_appl_level]
                                            if prot_level_exist:
                                                if protlevel == 'normal':
                                                    ret_list.append(
                                                        ('exported_provider_normal_app', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                    exp_count[cnt_id] = exp_count[
                                                        cnt_id] + 1
                                                elif protlevel == 'dangerous':
                                                    ret_list.append(
                                                        ('exported_provider_danger_appl', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                    exp_count[cnt_id] = exp_count[
                                                        cnt_id] + 1
                                                elif protlevel == 'signature':
                                                    ret_list.append(
                                                        ('exported_provider_signature_appl', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                elif protlevel == 'signatureOrSystem':
                                                    ret_list.append(
                                                        ('exported_provider_signatureorsystem_app', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            else:
                                                ret_list.append(
                                                    ('exported_provider_unknown_app', (itemname, item, perm), (an_or_a, itemname)))
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                    # Esteve 29.07.2016 - end
                                    # Esteve 08.08.2016 - begin - If the content provider does not target an API version lower than 17, it could still be exported by default, depending
                                    # on the API version of the platform. If it was below 17, the content
                                    # provider would be exported by default.
                                else:
                                    if itemname == 'Content Provider' and int(man_data_dic['target_sdk']) >= 17:
                                        perm = ''
                                        item = node.getAttribute(
                                            f'{ns}:name')
                                        item = escape_manifest_attribute(item)
                                        if node.getAttribute(f'{ns}:permission'):
                                            # permission exists
                                            perm = ('<strong>Permission: </strong>'
                                                    + node.getAttribute(f'{ns}:permission'))
                                            is_perm_exist = True
                                        if is_perm_exist:
                                            prot = ''
                                            if node.getAttribute(f'{ns}:permission') in permission_dict:
                                                prot = ('</br><strong>protectionLevel: </strong>'
                                                        + permission_dict[node.getAttribute(f'{ns}:permission')])
                                                prot_level_exist = True
                                                protlevel = permission_dict[
                                                    node.getAttribute(f'{ns}:permission')]
                                            if prot_level_exist:
                                                if protlevel == 'normal':
                                                    ret_list.append(
                                                        ('exported_provider_normal_new', (itemname, item, perm + prot), (itemname)))
                                                    exp_count[cnt_id] = exp_count[
                                                        cnt_id] + 1
                                                if protlevel == 'dangerous':
                                                    ret_list.append(
                                                        ('exported_provider_danger_new', (itemname, item, perm + prot), (itemname)))
                                                    exp_count[cnt_id] = exp_count[
                                                        cnt_id] + 1
                                                if protlevel == 'signature':
                                                    ret_list.append(
                                                        ('exported_provider_signature_new', (itemname, item, perm + prot), (itemname)))
                                                if protlevel == 'signatureOrSystem':
                                                    ret_list.append(
                                                        ('exported_provider_signatureorsystem_new', (itemname, item, perm + prot), (an_or_a, itemname)))
                                            else:
                                                ret_list.append(
                                                    ('exported_provider_unknown_new', (itemname, item, perm), (itemname)))
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                        else:
                                            if perm_appl_level_exists is False:
                                                ret_list.append(
                                                    ('exported_provider_2', (itemname, item), (an_or_a, itemname)))
                                                exp_count[cnt_id] = exp_count[
                                                    cnt_id] + 1
                                            else:
                                                perm = '<strong>Permission: </strong>' + perm_appl_level
                                                prot = ''
                                                if perm_appl_level in permission_dict:
                                                    prot = ('</br><strong>protectionLevel: </strong>'
                                                            + permission_dict[perm_appl_level])
                                                    prot_level_exist = True
                                                    protlevel = permission_dict[
                                                        perm_appl_level]
                                                if prot_level_exist:
                                                    if protlevel == 'normal':
                                                        ret_list.append(
                                                            ('exported_provider_normal_app_new', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                        exp_count[cnt_id] = exp_count[
                                                            cnt_id] + 1
                                                    elif protlevel == 'dangerous':
                                                        ret_list.append(
                                                            ('exported_provider_danger_app_new', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                        exp_count[cnt_id] = exp_count[
                                                            cnt_id] + 1
                                                    elif protlevel == 'signature':
                                                        ret_list.append(
                                                            ('exported_provider_signature_app_new', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                    elif protlevel == 'signatureOrSystem':
                                                        ret_list.append(
                                                            ('exported_provider_signatureorsystem_app_new', (itemname, item, perm + prot), (an_or_a, itemname)))
                                                else:
                                                    ret_list.append(
                                                        ('exported_provider_unknown_app_new', (itemname, item, perm), (an_or_a, itemname)))
                                                    exp_count[cnt_id] = exp_count[
                                                        cnt_id] + 1
                                    # Esteve 08.08.2016 - end

        # GRANT-URI-PERMISSIONS
        for granturi in granturipermissions:
            if granturi.getAttribute(f'{ns}:pathPrefix') == '/':
                ret_list.append(
                    ('improper_provider_permission', ('pathPrefix=/',), ()))
            elif granturi.getAttribute(f'{ns}:path') == '/':
                ret_list.append(('improper_provider_permission', ('path=/',), ()))
            elif granturi.getAttribute(f'{ns}:pathPattern') == '*':
                ret_list.append(('improper_provider_permission', ('path=*',), ()))
        # DATA
        for data in data_tag:
            if data.getAttribute(f'{ns}:scheme') == 'android_secret_code':
                xmlhost = data.getAttribute(f'{ns}:host')
                xmlhost = escape_manifest_attribute(xmlhost)
                ret_list.append(('dialer_code_found', (xmlhost,), ()))

            elif data.getAttribute(f'{ns}:port'):
                dataport = data.getAttribute(f'{ns}:port')
                dataport = escape_manifest_attribute(dataport)
                ret_list.append(('sms_receiver_port_found', (dataport,), ()))
        # INTENTS
        processed_priorities = {}
        for intent in intents:
            if intent.getAttribute(f'{ns}:priority').isdigit():
                value = intent.getAttribute(f'{ns}:priority')
                if int(value) > 100:
                    if value not in processed_priorities:
                        processed_priorities[value] = 1
                    else:
                        processed_priorities[value] += 1
        for priority, count in processed_priorities.items():
            ret_list.append(
                ('high_intent_priority_found', (priority, count,), ()))
        # ACTIONS
        for action in actions:
            if action.getAttribute(f'{ns}:priority').isdigit():
                value = action.getAttribute(f'{ns}:priority')
                if int(value) > 100:
                    ret_list.append(
                        ('high_action_priority_found', (value,), ()))
        for a_key, t_name, t_desc in ret_list:
            a_template = android_manifest_desc.MANIFEST_DESC.get(a_key)
            if a_template:
                ret_value.append({
                    'rule': a_key,
                    'title': a_template['title'] % t_name,
                    'severity': a_template['level'],
                    'description': a_template['description'] % t_desc,
                    'name': a_template['name'] % t_name,
                    'component': t_name,
                })
            else:
                logger.warning("No template found for key '%s'", a_key)

        for category in man_data_dic['categories']:
            if category == 'android.intent.category.LAUNCHER':
                break

        permissions = {}
        for k, permission in man_data_dic['perm'].items():
            permissions[k] = (
                {
                    'status': permission[0],
                    'info': permission[1],
                    'description': permission[2],
                })
        # Prepare return dict
        exported_comp = {
            'exported_activities': exp_count['act'],
            'exported_services': exp_count['ser'],
            'exported_receivers': exp_count['bro'],
            'exported_providers': exp_count['cnt'],
        }
        man_an_dic = {
            'manifest_anal': ret_value,
            'exported_act': exported,
            'exported_cnt': exported_comp,
            'browsable_activities': browsable_activities,
            'permissions': permissions,
            'network_security': network_security.analysis(
                checksum,
                app_dir,
                do_netsec,
                debuggable,
                src_type),
        }
        return man_an_dic
    except Exception as exp:
        msg = 'Error Performing Manifest Analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))



# -*- coding: utf_8 -*-
"""Android Static Code Analysis."""

import logging
from pathlib import Path

from django.conf import settings
from django.template.defaulttags import register

from mobsf.MobSF.utils import (
    android_component,
    append_scan_status,
    is_md5,
    key,
    pathify,
    print_n_send_error_response,
    relative_path,
)
from mobsf.StaticAnalyzer.models import (
    RecentScansDB,
)
from mobsf.StaticAnalyzer.views.android.xapk import (
    handle_aab,
    handle_split_apk,
    handle_xapk,
)
from mobsf.StaticAnalyzer.views.android.apk import (
    apk_analysis,
    src_analysis,
)
from mobsf.StaticAnalyzer.views.android.jar_aar import (
    aar_analysis,
    jar_analysis,
)
from mobsf.StaticAnalyzer.views.android.so import (
    so_analysis,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)

APK_TYPE = 'apk'
logger = logging.getLogger(__name__)
register.filter('key', key)
register.filter('android_component', android_component)
register.filter('relative_path', relative_path)
register.filter('pathify', pathify)


@login_required
def static_analyzer(request, checksum, api=False):
    """Do static analysis on an request and save to db."""
    try:
        rescan = False
        if api:
            re_scan = request.POST.get('re_scan', 0)
        else:
            re_scan = request.GET.get('rescan', 0)
        if re_scan == '1':
            rescan = True
        # Input validation
        app_dic = {}
        if not is_md5(checksum):
            return print_n_send_error_response(
                request,
                'Invalid Hash',
                api)
        robj = RecentScansDB.objects.filter(MD5=checksum)
        if not robj.exists():
            return print_n_send_error_response(
                request,
                'The file is not uploaded/available',
                api)
        typ = robj[0].SCAN_TYPE
        filename = robj[0].FILE_NAME
        allowed_exts = tuple(f'.{i}' for i in settings.ANDROID_EXTS)
        if (not filename.lower().endswith(allowed_exts)
                or typ not in settings.ANDROID_EXTS):
            return print_n_send_error_response(
                request,
                'Invalid file extension or file type',
                api)
        app_dic['dir'] = Path(settings.BASE_DIR)  # BASE DIR
        app_dic['app_name'] = filename  # APP ORIGINAL NAME
        app_dic['md5'] = checksum  # MD5
        msg = f'Scan Hash: {checksum}'
        logger.info(msg)
        # APP DIRECTORY
        app_dic['app_dir'] = Path(settings.UPLD_DIR) / checksum
        app_dic['tools_dir'] = app_dic['dir'] / 'StaticAnalyzer' / 'tools'
        app_dic['tools_dir'] = app_dic['tools_dir'].as_posix()
        app_dic['icon_path'] = ''
        msg = f'Starting Analysis on: {filename}'
        logger.info(msg)
        if typ == 'xapk':
            # Handle XAPK
            # Base APK will have the MD5 of XAPK
            if not handle_xapk(app_dic):
                raise Exception('Invalid XAPK File')
            typ = APK_TYPE
        elif typ == 'apks':
            # Handle Split APK
            if not handle_split_apk(app_dic):
                raise Exception('Invalid Split APK File')
            typ = APK_TYPE
        elif typ == 'aab':
            # Convert AAB to APK
            if not handle_aab(app_dic):
                raise Exception('Invalid AAB File')
            typ = APK_TYPE
        # Route to respective analysis
        if typ == APK_TYPE:
            return apk_analysis(request, app_dic, rescan, api)
        elif typ == 'jar':
            return jar_analysis(request, app_dic, rescan, api)
        elif typ == 'aar':
            return aar_analysis(request, app_dic, rescan, api)
        elif typ == 'so':
            return so_analysis(request, app_dic, rescan, api)
        elif typ == 'zip':
            return src_analysis(request, app_dic, rescan, api)
        else:
            err = ('Only APK, JAR, AAR, SO and Zipped '
                   'Android/iOS Source code supported now!')
            logger.error(err)
            append_scan_status(checksum, err)
            raise Exception(err)
    except Exception as exp:
        errmsg = 'Error Performing Static Analysis'
        logger.exception(errmsg)
        exp = exp.__doc__
        append_scan_status(checksum, errmsg, repr(exp))
        return print_n_send_error_response(request, repr(exp), api, exp)



# -*- coding: utf_8 -*-
"""Common Frida Server Update Management for Android and iOS."""

import logging
from pathlib import Path
from lzma import LZMAFile
from shutil import copyfileobj

import requests

from django.conf import settings

from mobsf.MobSF.utils import (
    is_internet_available,
    upstream_proxy,
)

logger = logging.getLogger(__name__)


class FridaServerUpdater:
    """Class for managing Frida server updates and downloads."""

    def __init__(self, platform, version):
        """Initialize the FridaServerUpdater.

        Args:
            platform (str): The platform ('android' or 'ios')
            version (str): The Frida version to manage
        """
        self.download_dir = Path(settings.DWD_DIR)
        self.platform = platform
        self.version = version

    def clean_up_old_binaries(self):
        """Delete old Frida server binaries."""
        if self.platform == 'android':
            file_pattern = 'frida-server*'
        else:
            file_pattern = 'frida_*'
        for f in self.download_dir.glob(file_pattern):
            if f.is_file() and self.version not in f.name:
                try:
                    f.unlink()
                except Exception:
                    pass

    def download_frida_server(self, url, fname, proxies, verify):
        """Download Frida server binary."""
        try:
            logger.info('Downloading Frida server v%s binary: %s', self.version, fname)
            dwd_loc = self.download_dir / fname

            with requests.get(
                    url,
                    timeout=15,
                    proxies=proxies,
                    verify=verify,
                    stream=True) as r:
                r.raise_for_status()

                with open(dwd_loc, 'wb') as f:
                    if fname.endswith('.deb'):
                        copyfileobj(r.raw, f)
                    else:
                        copyfileobj(LZMAFile(r.raw), f)

            self.clean_up_old_binaries()
            return True

        except Exception:
            logger.exception(
                '[ERROR] Downloading Frida Server v%s Binary', self.version)
            # Clean up partial download
            try:
                dwd_loc.unlink()
            except Exception:
                pass
        return False

    def update_frida_server(self, arch):
        """Update/download Frida server for the given architecture."""
        if self.platform == 'android':
            fserver = f'frida-server-{self.version}-{self.platform}-{arch}'
        else:
            fserver = f'frida_{self.version}_{self.platform}-{arch}.deb'
        frida_bin = self.download_dir / fserver
        if frida_bin.is_file():
            return True
        if not is_internet_available():
            return False
        try:
            proxies, verify = upstream_proxy('https')
        except Exception:
            logger.exception('[ERROR] Setting upstream proxy')
            proxies, verify = None, True

        try:
            # Get Frida release asset urls
            response = requests.get(
                f'{settings.FRIDA_SERVER}{self.version}',
                timeout=5,
                proxies=proxies,
                verify=verify,
            )
            response.raise_for_status()

            # Find the correct binary
            if self.platform == 'android':
                asset = f'{fserver}.xz'
            else:
                asset = fserver
            for item in response.json()['assets']:
                if item['name'] == asset:
                    return self.download_frida_server(
                        item['browser_download_url'], fserver, proxies, verify,
                    )

            logger.error(
                'Frida server v%s binary not found for platform: %s, architecture: %s',
                self.version,
                self.platform, arch,
            )

        except Exception:
            logger.exception('[ERROR] Fetching Frida Server v%s Release', self.version)

        return False



"""
WSGI config for MobSF project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/dev/howto/deployment/wsgi/
"""
import os
import warnings

from django.core.wsgi import get_wsgi_application

from whitenoise import WhiteNoise

from . import settings


warnings.filterwarnings('ignore', category=UserWarning, module='cffi')
os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'mobsf.MobSF.settings')

static = os.path.join(settings.BASE_DIR, 'static')
application = WhiteNoise(get_wsgi_application(),
                         root=static, prefix='static/')



# -*- coding: utf_8 -*-
"""REST API Middleware."""
from hmac import compare_digest

from django.http import JsonResponse
from django.utils.deprecation import MiddlewareMixin
from django.conf import settings

from mobsf.MobSF.init import api_key

OK = 200


def make_api_response(data, status=OK):
    """Make API Response."""
    resp = JsonResponse(
        data=data,
        status=status,
        safe=False)
    resp['Access-Control-Allow-Origin'] = '*'
    resp['Access-Control-Allow-Methods'] = 'POST'
    resp['Access-Control-Allow-Headers'] = 'Authorization, X-Mobsf-Api-Key'
    resp['Content-Type'] = 'application/json; charset=utf-8'
    return resp


def api_auth(meta):
    """Check if API Key Matches."""
    mobsf_api_key = api_key(settings.MOBSF_HOME)
    if 'HTTP_X_MOBSF_API_KEY' in meta:
        return compare_digest(mobsf_api_key, meta['HTTP_X_MOBSF_API_KEY'])
    elif 'HTTP_AUTHORIZATION' in meta:
        return compare_digest(mobsf_api_key, meta['HTTP_AUTHORIZATION'])
    return False


class RestApiAuthMiddleware(MiddlewareMixin):
    """
    Middleware.

    Middleware for REST API.
    """

    def process_request(self, request):
        """Middleware to handle API Auth."""
        if not request.path.startswith('/api/'):
            return
        if request.method == 'OPTIONS':
            return make_api_response({}, 200)
        if not api_auth(request.META):
            return make_api_response(
                {'error': 'You are unauthorized to make this request.'}, 401)



"""Initialize on first run."""
import logging
import os
import random
import subprocess
import sys
import shutil
import threading
from hashlib import sha256
from pathlib import Path
from importlib import (
    machinery,
    util,
)

from mobsf.MobSF.tools_download import install_jadx
from mobsf.install.windows.setup import windows_config_local

logger = logging.getLogger(__name__)

VERSION = '4.5.1'
BANNER = r"""
  __  __       _    ____  _____       _  _    ____  
 |  \/  | ___ | |__/ ___||  ___|_   _| || |  | ___| 
 | |\/| |/ _ \| '_ \___ \| |_  \ \ / / || |_ |___ \ 
 | |  | | (_) | |_) |__) |  _|  \ V /|__   _| ___) |
 |_|  |_|\___/|_.__/____/|_|     \_/    |_|(_)____/ 
"""  # noqa: W291
# ASCII Font: Standard


def first_run(secret_file, base_dir, mobsf_home):
    # Based on https://gist.github.com/ndarville/3452907#file-secret-key-gen-py
    base_dir = Path(base_dir)
    mobsf_home = Path(mobsf_home)
    secret_file = Path(secret_file)
    if os.getenv('MOBSF_SECRET_KEY'):
        secret_key = os.environ['MOBSF_SECRET_KEY']
    elif secret_file.exists() and secret_file.is_file():
        secret_key = secret_file.read_text().strip()
    else:
        try:
            secret_key = get_random()
            secret_file.write_text(secret_key)
        except IOError:
            raise Exception('Secret file generation failed' % secret_file)
        # Run Once
        make_migrations(base_dir)
        migrate(base_dir)
        # Install JADX
        thread = threading.Thread(
            target=install_jadx,
            name='install_jadx',
            args=(mobsf_home.as_posix(),))
        thread.start()
        # Windows Setup
        windows_config_local(mobsf_home.as_posix())
    return secret_key


def create_user_conf(mobsf_home, base_dir):
    try:
        config_path = mobsf_home / 'config.py'
        if not config_path.exists():
            sample_conf = base_dir / 'MobSF' / 'settings.py'
            dat = sample_conf.read_text().splitlines()
            config = []
            add = False
            for line in dat:
                if '^CONFIG-START^' in line:
                    add = True
                if '^CONFIG-END^' in line:
                    break
                if add:
                    config.append(line.lstrip())
            config.pop(0)
            conf_str = '\n'.join(config)
            config_path.write_text(conf_str)
    except Exception:
        logger.exception('Cannot create config file')


def django_operation(cmds, base_dir):
    """Generic Function for Djano operations."""
    manage = base_dir.parent / 'manage.py'
    if manage.exists() and manage.is_file():
        # Bail out for package
        return
    print(manage)
    args = [sys.executable, manage.as_posix()]
    args.extend(cmds)
    subprocess.call(args)


def make_migrations(base_dir):
    """Create Database Migrations."""
    try:
        django_operation(['makemigrations'], base_dir)
        django_operation(['makemigrations', 'StaticAnalyzer'], base_dir)
    except Exception:
        logger.exception('Cannot Make Migrations')


def migrate(base_dir):
    """Migrate Database."""
    try:
        django_operation(['migrate'], base_dir)
        django_operation(['migrate', '--run-syncdb'], base_dir)
        django_operation(['create_roles'], base_dir)
    except Exception:
        logger.exception('Cannot Migrate')


def get_random():
    choice = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    return ''.join([random.SystemRandom().choice(choice) for i in range(50)])


def get_mobsf_home(use_home, base_dir):
    try:
        base_dir = Path(base_dir)
        mobsf_home = ''
        if use_home:
            mobsf_home = Path.home() / '.MobSF'
            custom_home = os.getenv('MOBSF_HOME_DIR')
            if custom_home:
                p = Path(custom_home)
                if p.exists() and p.is_absolute() and p.is_dir():
                    mobsf_home = p
            # MobSF Home Directory
            if not mobsf_home.exists():
                mobsf_home.mkdir(parents=True, exist_ok=True)
            create_user_conf(mobsf_home, base_dir)
        else:
            mobsf_home = base_dir
        # Download Directory
        dwd_dir = mobsf_home / 'downloads'
        dwd_dir.mkdir(parents=True, exist_ok=True)
        # Screenshot Directory
        screen_dir = mobsf_home / 'screen'
        screen_dir.mkdir(parents=True, exist_ok=True)
        # Upload Directory
        upload_dir = mobsf_home / 'uploads'
        upload_dir.mkdir(parents=True, exist_ok=True)
        # Downloaded tools
        downloaded_tools_dir = mobsf_home / 'tools'
        downloaded_tools_dir.mkdir(parents=True, exist_ok=True)
        # Signatures Directory
        sig_dir = mobsf_home / 'signatures'
        sig_dir.mkdir(parents=True, exist_ok=True)
        if use_home:
            src = Path(base_dir) / 'signatures'
            try:
                shutil.copytree(src, sig_dir, dirs_exist_ok=True)
            except Exception:
                pass
        return mobsf_home.as_posix()
    except Exception:
        logger.exception('Creating MobSF Home Directory')


def get_mobsf_version():
    return BANNER, VERSION, f'v{VERSION}'


def load_source(modname, filename):
    loader = machinery.SourceFileLoader(modname, filename)
    spec = util.spec_from_file_location(modname, filename, loader=loader)
    module = util.module_from_spec(spec)
    loader.exec_module(module)
    return module


def get_docker_secret_by_file(secret_key):
    try:
        secret_path = os.environ.get(secret_key)
        path = Path(secret_path)
        if path.exists() and path.is_file():
            return path.read_text().strip()
    except Exception:
        logger.exception('Cannot read secret from %s', secret_path)
    raise Exception('Cannot read secret from file')


def get_secret_from_file_or_env(env_secret_key):
    docker_secret_key = f'{env_secret_key}_FILE'
    if os.environ.get(docker_secret_key):
        return get_docker_secret_by_file(docker_secret_key)
    else:
        return os.environ[env_secret_key]


def api_key(home_dir):
    """Print REST API Key."""
    # Form Docker Secrets
    if os.environ.get('MOBSF_API_KEY_FILE'):
        logger.info('\nAPI Key read from docker secrets')
        try:
            return get_docker_secret_by_file('MOBSF_API_KEY_FILE')
        except Exception:
            logger.exception('Cannot read API Key from docker secrets')
    # From Environment Variable
    if os.environ.get('MOBSF_API_KEY'):
        logger.info('\nAPI Key read from environment variable')
        return os.environ['MOBSF_API_KEY']
    home_dir = Path(home_dir)
    secret_file = home_dir / 'secret'
    if secret_file.exists() and secret_file.is_file():
        try:
            _api_key = secret_file.read_bytes().strip()
            return sha256(_api_key).hexdigest()
        except Exception:
            logger.exception('Cannot Read API Key')
    return None



# -*- coding: utf_8 -*-
"""
Shared Functions.

PDF Generation
"""
import json
import logging
import os
import platform

from django.http import HttpResponse
from django.template.loader import get_template

import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal
from mobsf.MobSF import settings
from mobsf.MobSF.utils import (
    is_md5,
    print_n_send_error_response,
    upstream_proxy,
)
from mobsf.StaticAnalyzer.models import (
    RecentScansDB,
    StaticAnalyzerAndroid,
    StaticAnalyzerIOS,
    StaticAnalyzerWindows,
)
from mobsf.StaticAnalyzer.views.common.appsec import (
    get_android_dashboard,
    get_ios_dashboard,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    get_avg_cvss,
)
from mobsf.StaticAnalyzer.views.android.db_interaction import (
    get_context_from_db_entry as adb)
from mobsf.StaticAnalyzer.views.ios.db_interaction import (
    get_context_from_db_entry as idb)
from mobsf.StaticAnalyzer.views.windows.db_interaction import (
    get_context_from_db_entry as wdb)
from mobsf.MobSF.views.authentication import (
    login_required,
)

logger = logging.getLogger(__name__)
try:
    import pdfkit
except ImportError:
    logger.warning(
        'wkhtmltopdf is not installed/configured properly.'
        ' PDF Report Generation is disabled')
logger = logging.getLogger(__name__)
ctype = 'application/json; charset=utf-8'


@login_required
def pdf(request, checksum, api=False, jsonres=False):
    try:
        if not is_md5(checksum):
            if api:
                return {'error': 'Invalid Hash'}
            else:
                return HttpResponse(
                    json.dumps({'md5': 'Invalid Hash'}),
                    content_type=ctype, status=500)
        # Do Lookups
        android_static_db = StaticAnalyzerAndroid.objects.filter(
            MD5=checksum)
        ios_static_db = StaticAnalyzerIOS.objects.filter(
            MD5=checksum)
        win_static_db = StaticAnalyzerWindows.objects.filter(
            MD5=checksum)

        if android_static_db.exists():
            context, template = handle_pdf_android(android_static_db)
        elif ios_static_db.exists():
            context, template = handle_pdf_ios(ios_static_db)
        elif win_static_db.exists():
            context, template = handle_pdf_win(win_static_db)
        else:
            if api:
                return {'report': 'Report not Found'}
            else:
                return HttpResponse(
                    json.dumps({'report': 'Report not Found'}),
                    content_type=ctype,
                    status=500)
        # Do VT Scan only on binaries
        context['virus_total'] = None
        ext = os.path.splitext(context['file_name'].lower())[1]
        if settings.VT_ENABLED and ext != '.zip':
            app_bin = os.path.join(
                settings.UPLD_DIR,
                checksum + '/',
                checksum + ext)
            vt = VirusTotal.VirusTotal(checksum)
            context['virus_total'] = vt.get_result(app_bin)
        # Get Local Base URL
        proto = 'file://'
        host_os = 'nix'
        if platform.system() == 'Windows':
            proto = 'file:///'
            host_os = 'windows'
        context['base_url'] = proto + settings.BASE_DIR
        context['dwd_dir'] = proto + settings.DWD_DIR
        context['host_os'] = host_os
        context['timestamp'] = RecentScansDB.objects.get(
            MD5=checksum).TIMESTAMP
        try:
            if api and jsonres:
                return {'report_dat': context}
            else:
                options = {
                    'page-size': 'Letter',
                    'quiet': '',
                    'enable-local-file-access': '',
                    'no-collate': '',
                    'margin-top': '0.50in',
                    'margin-right': '0.50in',
                    'margin-bottom': '0.50in',
                    'margin-left': '0.50in',
                    'encoding': 'UTF-8',
                    'orientation': 'Landscape',
                    'custom-header': [
                        ('Accept-Encoding', 'gzip'),
                    ],
                    'no-outline': None,
                }
                # Added proxy support to wkhtmltopdf
                proxies, _ = upstream_proxy('https')
                if proxies['https']:
                    options['proxy'] = proxies['https']
                html = template.render(context)
                pdf_dat = pdfkit.from_string(html, False, options=options)
                if api:
                    return {'pdf_dat': pdf_dat}
                return HttpResponse(pdf_dat,
                                    content_type='application/pdf')
        except Exception as exp:
            logger.exception('Error Generating PDF Report')
            if api:
                return {
                    'error': 'Cannot Generate PDF/JSON',
                    'err_details': str(exp)}
            else:
                err = {
                    'pdf_error': 'Cannot Generate PDF',
                    'err_details': str(exp)}
                return HttpResponse(
                    json.dumps(err),  # lgtm [py/stack-trace-exposure]
                    content_type=ctype,
                    status=500)
    except Exception as exp:
        logger.exception('Error Generating PDF Report')
        msg = str(exp)
        exp = exp.__doc__
        if api:
            return print_n_send_error_response(request, msg, True, exp)
        else:
            return print_n_send_error_response(request, msg, False, exp)


def handle_pdf_android(static_db):
    logger.info(
        'Fetching data from DB for '
        'PDF Report Generation (Android)')
    context = adb(static_db)
    context['average_cvss'] = get_avg_cvss(
        context['code_analysis'])
    context['appsec'] = get_android_dashboard(static_db)
    if context['file_name'].lower().endswith('.zip'):
        logger.info('Generating PDF report for android zip')
    else:
        logger.info('Generating PDF report for android apk')
    return context, get_template('pdf/android_report.html')


def handle_pdf_ios(static_db):
    logger.info('Fetching data from DB for '
                'PDF Report Generation (IOS)')
    context = idb(static_db)
    context['appsec'] = get_ios_dashboard(static_db)
    if context['file_name'].lower().endswith('.zip'):
        logger.info('Generating PDF report for IOS zip')
        context['average_cvss'] = get_avg_cvss(
            context['code_analysis'])
    else:
        logger.info('Generating PDF report for IOS ipa')
        context['average_cvss'] = get_avg_cvss(
            context['binary_analysis'])
    return context, get_template('pdf/ios_report.html')


def handle_pdf_win(static_db):
    logger.info(
        'Fetching data from DB for '
        'PDF Report Generation (APPX)')
    context = wdb(static_db)
    return context, get_template('pdf/windows_report.html')



# !/usr/bin/python
# coding=utf-8
import shutil
import subprocess

import lief

from mobsf.StaticAnalyzer.views.common.binary.strings import (
    strings_on_binary,
)
from mobsf.MobSF.utils import (
    run_with_timeout,
)

from django.conf import settings


NA = 'Not Applicable'
NO_RELRO = 'No RELRO'
PARTIAL_RELRO = 'Partial RELRO'
FULL_RELRO = 'Full RELRO'
INFO = 'info'
WARNING = 'warning'
HIGH = 'high'


def nm_is_debug_symbol_stripped(elf_file):
    """Check if debug symbols are stripped using OS utility."""
    # https://linux.die.net/man/1/nm
    out = subprocess.check_output(
        [shutil.which('nm'), '--debug-syms', elf_file],
        stderr=subprocess.STDOUT)
    return b'no debug symbols' in out


class ELFChecksec:
    def __init__(self, elf_file, so_rel):
        self.elf_path = elf_file.as_posix()
        self.elf_rel = so_rel
        self.elf = run_with_timeout(
            lief.parse,
            settings.BINARY_ANALYSIS_TIMEOUT,
            self.elf_path)

    def checksec(self):
        elf_dict = {}
        elf_dict['name'] = self.elf_rel
        if not self.is_elf(self.elf_path):
            return
        is_nx = self.is_nx()
        if is_nx:
            severity = INFO
            desc = (
                'The binary has NX bit set. This marks a '
                'memory page non-executable making attacker '
                'injected shellcode non-executable.')
        else:
            severity = HIGH
            desc = (
                'The binary does not have NX bit set. NX bit '
                'offer protection against exploitation of memory corruption '
                'vulnerabilities by marking memory page as non-executable. '
                'Use option --noexecstack or -z noexecstack to mark stack as '
                'non executable.')
        elf_dict['nx'] = {
            'is_nx': is_nx,
            'severity': severity,
            'description': desc,
        }
        severity = 'info'
        is_pie = self.is_pie()
        if is_pie == 'dso':
            is_pie = 'Dynamic Shared Object (DSO)'
            desc = (
                'The shared object is build with -fPIC flag which '
                'enables Position independent code. This makes Return '
                'Oriented Programming (ROP) attacks much more difficult '
                'to execute reliably.')
        elif is_pie == 'pie':
            is_pie = 'Position Independent Executable (PIE)'
            desc = (
                'The shared object is build with -fPIC flag which '
                'enables Position independent code. This makes Return '
                'Oriented Programming (ROP) attacks much more difficult '
                'to execute reliably.')
        elif is_pie == 'rel':
            is_pie = 'Relocatable Object File'
            desc = (
                'The shared object is build with -fPIC flag which '
                'enables Position independent code. This makes Return '
                'Oriented Programming (ROP) attacks much more difficult '
                'to execute reliably.')
        elif is_pie == 'no':
            is_pie = 'No PIE'
            severity = 'high'
            desc = (
                'The shared object is built without Position '
                'Independent Code flag. In order to prevent '
                'an attacker from reliably jumping to, for example, a '
                'particular exploited function in memory, Address '
                'space layout randomization (ASLR) randomly arranges '
                'the address space positions of key data areas of a '
                'process, including the base of the executable and the '
                'positions of the stack,heap and libraries. Use compiler '
                'option -fPIC to enable Position Independent Code.')
        elf_dict['pie'] = {
            'is_pie': is_pie,
            'severity': severity,
            'description': desc,
        }
        has_canary = self.has_canary()
        if has_canary:
            severity = INFO
            desc = (
                'This binary has a stack canary value '
                'added to the stack so that it will be overwritten by '
                'a stack buffer that overflows the return address. '
                'This allows detection of overflows by verifying the '
                'integrity of the canary before function return.')
        else:
            severity = HIGH
            desc = (
                'This binary does not have a stack '
                'canary value added to the stack. Stack canaries '
                'are used to detect and prevent exploits from '
                'overwriting return address. Use the option '
                '-fstack-protector-all to enable stack canaries. '
                'Not applicable for Dart/Flutter libraries unless '
                'Dart FFI is used.')
        elf_dict['stack_canary'] = {
            'has_canary': has_canary,
            'severity': severity,
            'description': desc,
        }
        relro = self.relro()
        if relro == NA:
            severity = INFO
            desc = ('RELRO checks are not applicable for '
                    'Flutter/Dart binaries')
        elif relro == FULL_RELRO:
            severity = INFO
            desc = (
                'This shared object has full RELRO '
                'enabled. RELRO ensures that the GOT cannot be '
                'overwritten in vulnerable ELF binaries. '
                'In Full RELRO, the entire GOT (.got and '
                '.got.plt both) is marked as read-only.')
        elif relro == PARTIAL_RELRO:
            severity = WARNING
            desc = (
                'This shared object has partial RELRO '
                'enabled. RELRO ensures that the GOT cannot be '
                'overwritten in vulnerable ELF binaries. '
                'In partial RELRO, the non-PLT part of the GOT '
                'section is read only but .got.plt is still '
                'writeable. Use the option -z,relro,-z,now to '
                'enable full RELRO.')
        else:
            severity = HIGH
            desc = (
                'This shared object does not have RELRO '
                'enabled. The entire GOT (.got and '
                '.got.plt both) are writable. Without this compiler '
                'flag, buffer overflows on a global variable can '
                'overwrite GOT entries. Use the option '
                '-z,relro,-z,now to enable full RELRO and only '
                '-z,relro to enable partial RELRO.')
        elf_dict['relocation_readonly'] = {
            'relro': relro,
            'severity': severity,
            'description': desc,
        }
        rpath = self.rpath()
        if rpath:
            severity = HIGH
            desc = (
                'The binary has RPATH set. In certain cases, '
                'an attacker can abuse this feature to run arbitrary '
                'libraries for code execution and privilege '
                'escalation. The only time a library should '
                'set RPATH is when it is linked to private '
                'libraries in the same package. Remove the '
                'compiler option -rpath to remove RPATH.')
            rpt = rpath.rpath
        else:
            severity = INFO
            desc = (
                'The binary does not have run-time search path '
                'or RPATH set.')
            rpt = rpath
        elf_dict['rpath'] = {
            'rpath': rpt,
            'severity': severity,
            'description': desc,
        }
        runpath = self.runpath()
        if runpath:
            severity = HIGH
            desc = (
                'The binary has RUNPATH set. In certain cases, '
                'an attacker can abuse this feature and or modify '
                'environment variables to run arbitrary '
                'libraries for code execution and privilege '
                'escalation. The only time a library should '
                'set RUNPATH is when it is linked to private '
                'libraries in the same package. Remove the compiler '
                'option --enable-new-dtags,-rpath to remove RUNPATH.')
            rnp = runpath.runpath
        else:
            severity = INFO
            desc = (
                'The binary does not have RUNPATH set.')
            rnp = runpath
        elf_dict['runpath'] = {
            'runpath': rnp,
            'severity': severity,
            'description': desc,
        }
        fortified_functions = self.fortify()
        if fortified_functions:
            severity = INFO
            desc = ('The binary has the '
                    f'following fortified functions: {fortified_functions}')
        else:
            if self.is_dart():
                severity = INFO
            else:
                severity = WARNING
            desc = ('The binary does not have any '
                    'fortified functions. Fortified functions '
                    'provides buffer overflow checks against '
                    'glibc\'s commons insecure functions like '
                    'strcpy, gets etc. Use the compiler option '
                    '-D_FORTIFY_SOURCE=2 to fortify functions. '
                    'This check is not applicable for '
                    'Dart/Flutter libraries.')
        elf_dict['fortify'] = {
            'is_fortified': bool(fortified_functions),
            'severity': severity,
            'description': desc,
        }
        is_stripped = self.is_symbols_stripped()
        if is_stripped:
            severity = INFO
            desc = 'Symbols are stripped.'
        else:
            severity = WARNING
            desc = 'Symbols are available.'
        elf_dict['symbol'] = {
            'is_stripped': is_stripped,
            'severity': severity,
            'description': desc,
        }
        return elf_dict

    def is_elf(self, elf_path):
        return lief.is_elf(elf_path)

    def is_nx(self):
        return self.elf.has_nx

    def is_pie(self):
        if self.elf.header.file_type == lief.ELF.Header.FILE_TYPE.DYN:
            if self.elf.has(lief.ELF.DynamicEntry.TAG.DEBUG_TAG):
                return 'pie'
            else:
                return 'dso'
        elif self.elf.header.file_type == lief.ELF.Header.FILE_TYPE.REL:
            return 'rel'
        return 'no'

    def is_dart(self):
        dart = ('_kDartVmSnapshotInstructions',
                'Dart_Cleanup')
        if any(i in self.strings() for i in dart):
            return True
        for symbol in dart:
            try:
                if self.elf.get_symbol(symbol):
                    return True
            except Exception:
                pass
        return False

    def has_canary(self):
        if self.is_dart():
            return True
        for symbol in ('__stack_chk_fail',
                       '__intel_security_cookie'):
            try:
                if self.elf.get_symbol(symbol):
                    return True
            except Exception:
                pass
        return False

    def relro(self):
        try:
            gnu_relro = lief.ELF.Segment.TYPE.GNU_RELRO
            bind_now_flag = lief.ELF.DynamicEntryFlags.FLAG.BIND_NOW
            flags_tag = lief.ELF.DynamicEntry.TAG.FLAGS
            flags1_tag = lief.ELF.DynamicEntry.TAG.FLAGS_1
            now_flag = lief.ELF.DynamicEntryFlags.FLAG.NOW

            if self.is_dart():
                return NA

            if not self.elf.get(gnu_relro):
                return NO_RELRO

            flags = self.elf.get(flags_tag)
            bind_now = flags and bind_now_flag in flags

            flags1 = self.elf.get(flags1_tag)
            now = flags1 and now_flag in flags1

            if bind_now or now:
                return FULL_RELRO
            else:
                return PARTIAL_RELRO
        except Exception:
            pass
        return NO_RELRO

    def rpath(self):
        rpath = lief.ELF.DynamicEntry.TAG.RPATH
        return self.elf.get(rpath)

    def runpath(self):
        runpath = lief.ELF.DynamicEntry.TAG.RUNPATH
        return self.elf.get(runpath)

    def is_symbols_stripped(self):
        try:
            for i in self.elf.symtab_symbols:
                if i:
                    return False
            return True
        except Exception:
            try:
                return nm_is_debug_symbol_stripped(
                    self.elf_path)
            except Exception:
                return True

    def fortify(self):
        fortified_funcs = []
        for function in self.elf.symbols:
            if isinstance(function.name, bytes):
                try:
                    function_name = function.name.decode('utf-8')
                except UnicodeDecodeError:
                    function_name = function.name.decode('utf-8', 'replace')
            else:
                function_name = function.name
            if function_name.endswith('_chk'):
                fortified_funcs.append(function.name)
        return fortified_funcs

    def strings(self):
        normalized = set()
        try:
            elf_strings = self.elf.strings
        except Exception:
            elf_strings = None
        if not elf_strings:
            elf_strings = strings_on_binary(self.elf_path)
        for i in elf_strings:
            if isinstance(i, bytes):
                continue
            normalized.add(i)
        return list(normalized)

    def get_symbols(self):
        symbols = []
        try:
            for i in self.elf.symtab_symbols:
                symbols.append(i.name)
        except Exception:
            pass
        return symbols



# -*- coding: utf_8 -*-
"""Extract packages from APK."""
import logging

from mobsf.MobSF.utils import (
    PKG_REGEX,
)


logger = logging.getLogger(__name__)


def merge_common_packages(items):
    """Merge common packages."""
    items = list(items)
    items.sort()  # Sort items lexicographically
    merged = []
    for item in items:
        if not merged or not item.startswith(merged[-1] + '.'):
            merged.append(item)
    return merged


def extract_packages(file_data):
    """Extract package names from file data."""
    packages = set()
    try:
        for item in file_data:
            # tuple has file path and file content
            # we are interested in file content's first line
            pkg = item[1].split('\n')[0]
            match = PKG_REGEX.search(pkg)
            if match and match.group(1) != '_COROUTINE':
                packages.add(match.group(1))
        packages = merge_common_packages(packages)
    except Exception:
        logger.exception('Extracting packages from file data')
    return sorted(packages)


def get_group_name(file_name, group):
    """Get group and name from file name."""
    parts = file_name.split('_')

    if parts and len(parts) == 2:
        group, name = parts[0], parts[1]
    else:
        name = file_name.replace('_', '-')
        if name.startswith('kotlinx-'):
            group = 'org.jetbrains.kotlinx'

    return group, name


def android_sbom(app_dir):
    """Extract SBOM from files."""
    sbom = set()
    for vfile in app_dir.rglob('*.version'):
        try:
            dependency = vfile.stem
            group, name = '', ''
            version = vfile.read_text().strip() or ''
            version = 'dynamic' if version.startswith('task') else version

            if '_' in dependency:
                group, name = get_group_name(dependency, group)
            sbom.add(f'{group}:{name}@{version}')
        except Exception:
            pass
    return sorted(sbom)


def sbom(app_dir, file_data):
    """Extract SBOM from version files and decompiled source code."""
    return {
        'sbom_versioned': android_sbom(app_dir),
        'sbom_packages': extract_packages(file_data),
    }



from django.core.management.base import BaseCommand

from django_q.models import (
    OrmQ,
    Task,
)

from mobsf.StaticAnalyzer.models import EnqueuedTask


class Command(BaseCommand):
    help = 'Deletes all tasks in Django Q'  # noqa: A003

    def handle(self, *args, **kwargs):
        Task.objects.all().delete()
        OrmQ.objects.all().delete()
        EnqueuedTask.objects.all().delete()
        self.stdout.write(self.style.SUCCESS('Successfully deleted all Django Q tasks'))



"""Command to create Authorization Roles."""
from django.core.management.base import BaseCommand

from mobsf.MobSF.views.authorization import create_authorization_roles


class Command(BaseCommand):
    help = 'Create Authorization Roles.'  # noqa: A003

    def handle(self, *args, **kwargs):
        create_authorization_roles()
        self.stdout.write('Roles Created Successfully!')
